"""
This file holds specs that are in the new core3 archives but that don't have an entry
in the meta_data directory. This happens when the client itself collects the data
and puts it into the archive.
"""
from functools import partial

from insights.core.context import SerializedArchiveContext
from insights.specs import Specs
from insights.core.spec_factory import RawFileProvider, simple_file

simple_file = partial(simple_file, context=SerializedArchiveContext)


class Core3Specs(Specs):
    ansible_host = simple_file("ansible_host")
    branch_info = simple_file("/branch_info", kind=RawFileProvider)
    display_name = simple_file("display_name")
    version_info = simple_file("version_info")



"""
LsUsrSbin - command ``ls -ln /usr/sbin``
========================================

The ``ls -ln /usr/sbin`` command provides information for the listing of the
``/usr/sbin`` directory.

Sample input is shown in the Examples. See ``FileListing`` class for
additional information.

For ls_usr_sbin, it may collect a lot of files or directories that may not be
necessary, so a default filter `add_filter(Specs.ls_usr_sbin, "total")` has
been added in this parser.

If addtional file or directory need to be collected by this parser, please
add related filter to corresponding code.

Sample added filter:

    >>> add_filter(Specs.ls_usr_sbin, "accessdb")

Sample directory list collected::

    total 41472
    -rwxr-xr-x. 1 0  0   11720 Mar 18  2014 accessdb

Examples:

    >>> "accessdb" in ls_usr_sbin
    False
    >>> "/usr/sbin" in ls_usr_sbin
    True
    >>> ls_usr_sbin.dir_entry('/usr/sbin', 'accessdb')['type']
    '-'

Sample added filter:

    >>> add_filter(Specs.ls_usr_sbin, "accessdb")
    >>> add_filter(Specs.ls_usr_sbin, "postdrop")

Sample directory list collected::

    total 41472
    -rwxr-xr-x. 1 0  0   11720 Mar 18  2014 accessdb
    -rwxr-sr-x. 1 0 90  218552 Jan 27  2014 postdrop

Examples:

    >>> "accessdb" in ls_usr_sbin
    False
    >>> "/usr/sbin" in ls_usr_sbin
    True
    >>> ls_usr_sbin.dir_entry('/usr/sbin', 'accessdb')['type']
    '-'
    >>> ls_usr_sbin.dir_entry('/usr/sbin', 'postdrop')['type']
    '-'
"""


from insights.core.filters import add_filter
from insights.specs import Specs

from .. import CommandParser, parser
from .. import FileListing


add_filter(Specs.ls_usr_sbin, "total")


@parser(Specs.ls_usr_sbin)
class LsUsrSbin(CommandParser, FileListing):
    """Parses output of ``ls -ln /usr/sbin`` command."""
    pass



"""
Nova configuration - file ``/etc/nova/nova.conf``
=================================================
"""
from insights.core import IniConfigFile
from insights.core.plugins import parser
from insights.specs import Specs


@parser(Specs.nova_conf)
class NovaConf(IniConfigFile):
    """
    The Nova configuration file is a standard '.ini' file and this parser uses
    the ``IniConfigFile`` class to read it.

    Sample configuration::

    [DEFAULT]
    notification_topics=notifications
    rpc_backend=rabbit
    use_ipv6=True
    notify_on_state_change=vm_and_task_state
    notify_api_faults=False
    state_path=/var/lib/nova
    report_interval = 10
    osapi_compute_listen=fd00:4888:1000:f901::c1
    osapi_compute_workers=32
    metadata_listen=fd00:4888:1000:f901::c1
    metadata_workers=32
    service_down_time=60
    rootwrap_config=/etc/nova/rootwrap.conf
    auth_strategy=keystone
    use_forwarded_for=False
    novncproxy_host=fd00:4888:1000:f901::c1
    novncproxy_port=6080
    network_api_class=nova.network.neutronv2.api.API
    security_group_api=neutron
    debug=False
    verbose=False
    log_dir=/var/log/nova
    use_syslog=False
    scheduler_host_manager=nova.scheduler.host_manager.HostManager
    scheduler_host_subset_size=1
    cpu_allocation_ratio=16.0
    disk_allocation_ratio=1.0
    max_io_ops_per_host=8
    max_instances_per_host=50
    ram_allocation_ratio=1.0
    scheduler_available_filters=nova.scheduler.filters.all_filters
    scheduler_weight_classes=nova.scheduler.weights.all_weighers
    scheduler_max_attempts=3
    vif_plugging_is_fatal=True
    vif_plugging_timeout=300
    firewall_driver=nova.virt.firewall.NoopFirewallDriver
    volume_api_class=nova.volume.cinder.API

    [ephemeral_storage_encryption]

    [glance]
    api_servers=http://[fd00:4888:1000:f901::a000]:9292

    [keystone_authtoken]
    auth_uri=http://[fd00:4888:1000:f901::a000]:5000/v2.0
    identity_uri=http://192.168.1.107:35357
    admin_user=nova
    admin_password=*********
    admin_tenant_name=service
    service_metadata_proxy=True
    ovs_bridge=br-int
    extension_sync_interval=600
    rabbit_use_ssl=False
    rabbit_userid=guest
    rabbit_password=*********
    rabbit_ha_queues=True
    heartbeat_timeout_threshold=60

    Examples:
        >>> type(conf)
        <class 'insights.parsers.nova_conf.NovaConf'>
        >>> conf.sections()
        ['ephemeral_storage_encryption', 'glance', 'keystone_authtoken']
        >>> conf.has_option('DEFAULT', 'use_ipv6')
        True
        >>> conf.get("DEFAULT", "log_dir")
        '/var/log/nova'
        >>> conf.get("DEFAULT", "debug")
        'False'
        >>> conf.getboolean("DEFAULT", "verbose")
        False
        >>> conf.getint("keystone_authtoken", "extension_sync_interval")
        600
    """
    pass



#!/usr/bin/env python
"""
Example Cluster Rule - ntp_compare
==================================
This rule can be run from the root directory of the insights-core
repo using the following command line::

    $ insights-run -p examples.cluster_rules.ntp_compare \
            -i topology_example \
            examples/cluster_rules/cluster_hosts.tar.gz

or from the examples/cluster_rules directory::

    $ ./ntp_compare.py -i topology_example cluster_hosts.tar.gz

"""
from insights import make_fail, make_pass
from insights.specs import Specs
from insights.core.cluster import ClusterMeta
from insights.core.plugins import fact, rule
from insights.util.fs import sha256

# Jinja template to use for make_pass
SUCCESS_TEMPLATE = """
Servers:
{% for n in servers %}
    {{ n }}
{% endfor %}

Nodes:
{% for n in nodes %}
    {{ n }}
{% endfor %}""".strip()

# The CONTENT variable provides the Jinja content templates that will
# be displayed depending upon which code is returned by make_pass/make_fail
CONTENT = {
    "DISTINCT_NTP_CONFS": "{{confs}} distinct NTP configurations of {{nodes}} members.",
    "MATCHING_NTP_CONFS": SUCCESS_TEMPLATE
}


@fact(Specs.ntp_conf)
def ntp_sha256(ntp):
    """ datasource object: Datasource for the ntp.conf spec """
    return {"sha": sha256(ntp.path)}


@rule(ntp_sha256, ClusterMeta, cluster=True)
def report(shas, meta):
    """
    Cluster rule to compare ntp.conf files across a cluster

    ``shas`` is a Pandas DataFrame for the facts for each host
    by the fact ``ntp_sha256``.  See
    https://pandas.pydata.org/pandas-docs/stable/api.html#dataframe
    for information on available attributes and methods.

    ``meta`` is a dictionary that contains the information from the
    cluster topology file provided by the ``-i`` switch.  The dictionary
    keys are the sections, and the values are a list of the host
    information provided in the toplolgy file.

    Arguments:
        shas (pandas.DataFrame): Includes facts from ``ntp_sha256``
            fact with column "sha" and one row per host in the cluster.
        meta (dict): Keys are the sections in the topology file and
            values are a list of the values in the section.
    """
    num_members = meta.num_members
    uniq = shas.sha.unique()
    if len(shas) != num_members or len(uniq) != 1:
        return make_fail("DISTINCT_NTP_CONFS", confs=len(uniq), nodes=num_members)

    return make_pass("MATCHING_NTP_CONFS", nodes=meta['nodes'], servers=meta['servers'])


if __name__ == "__main__":
    from insights import run
    run(report, print_summary=True)



"""
Custom datasources for ps information
"""
from insights.core.context import HostContext
from insights.core.dr import SkipComponent
from insights.core.plugins import datasource
from insights.core.spec_factory import DatasourceProvider, simple_command
from insights.specs import Specs


class LocalSpecs(Specs):
    """ Local specs used only by ps datasources """

    ps_eo_args = simple_command("/bin/ps -eo pid,args")
    """ Returns ps output including pid and full args """


@datasource(LocalSpecs.ps_eo_args, HostContext)
def ps_eo_cmd(broker):
    """
    Custom datasource to collect the full paths to all running commands on the system
    provided by the ``ps -eo pid,args`` command.  After collecting the data, all of the
    args are trimmed to leave only the command including full path.

    Sample output from the ``ps -eo pid, args`` command::

        PID COMMAND
          1 /usr/lib/systemd/systemd --switched-root --system --deserialize 31
          2 [kthreadd]
          3 [rcu_gp]
          4 [rcu_par_gp]
          6 [kworker/0:0H-events_highpri]
          9 [mm_percpu_wq]
         10 [rcu_tasks_kthre]
         11 /usr/bin/python3 /home/user1/python_app.py
         12 [kworker/u16:0-kcryptd/253:0]

    This datasource trims off the args to minimize possible PII and sensitive information.
    After trimming the data looks like this::

        PID COMMAND
          1 /usr/lib/systemd/systemd
          2 [kthreadd]
          3 [rcu_gp]
          4 [rcu_par_gp]
          6 [kworker/0:0H-events_highpri]
          9 [mm_percpu_wq]
         10 [rcu_tasks_kthre]
         11 /usr/bin/python3
         12 [kworker/u16:0-kcryptd/253:0]

    Returns:
        str: Returns a multiline string in the same format as ``ps`` output

    Raises:
        SkipComponent: Raised if no data is available
    """
    content = broker[LocalSpecs.ps_eo_args].content
    data = []
    data.append('PID COMMAND')
    start = False
    for l in content:
        if 'PID' in l and 'COMMAND' in l:
            start = True
            continue
        if not start:
            continue
        pid, args = l.strip().split(None, 1)
        if ' ' in args:
            cmd, _ = args.split(None, 1)
        else:
            cmd = args
        data.append('{0} {1}'.format(pid, cmd))

    if len(data) > 1:
        return DatasourceProvider('\n'.join(data), relative_path='insights_commands/ps_eo_cmd')

    raise SkipComponent()



from insights.core.spec_factory import SpecSet, RegistryPoint


class Specs(SpecSet):
    abrt_ccpp_conf = RegistryPoint(filterable=True)
    abrt_status_bare = RegistryPoint()
    alternatives_display_python = RegistryPoint()
    amq_broker = RegistryPoint(multi_output=True)
    ansible_host = RegistryPoint()
    auditctl_status = RegistryPoint()
    auditd_conf = RegistryPoint()
    audit_log = RegistryPoint(filterable=True)
    autofs_conf = RegistryPoint()
    avc_hash_stats = RegistryPoint()
    avc_cache_threshold = RegistryPoint()
    aws_instance_id_doc = RegistryPoint()
    aws_instance_id_pkcs7 = RegistryPoint()
    aws_instance_type = RegistryPoint()
    awx_manage_check_license = RegistryPoint()
    awx_manage_check_license_data = RegistryPoint(filterable=True)
    awx_manage_print_settings = RegistryPoint()
    azure_instance_type = RegistryPoint()
    azure_instance_plan = RegistryPoint()
    bios_uuid = RegistryPoint()
    blkid = RegistryPoint()
    bond = RegistryPoint(multi_output=True)
    bond_dynamic_lb = RegistryPoint(multi_output=True)
    boot_loader_entries = RegistryPoint(multi_output=True)
    branch_info = RegistryPoint(raw=True)
    brctl_show = RegistryPoint()
    candlepin_broker = RegistryPoint()
    candlepin_error_log = RegistryPoint(filterable=True)
    candlepin_log = RegistryPoint(filterable=True)
    cdc_wdm = RegistryPoint()
    checkin_conf = RegistryPoint()
    catalina_out = RegistryPoint(multi_output=True, filterable=True)
    catalina_server_log = RegistryPoint(multi_output=True, filterable=True)
    cciss = RegistryPoint(multi_output=True)
    ceilometer_central_log = RegistryPoint(filterable=True)
    ceilometer_collector_log = RegistryPoint(filterable=True)
    ceilometer_compute_log = RegistryPoint(filterable=True)
    ceilometer_conf = RegistryPoint()
    ceph_conf = RegistryPoint(filterable=True)
    ceph_config_show = RegistryPoint(multi_output=True)
    ceph_df_detail = RegistryPoint()
    ceph_health_detail = RegistryPoint()
    ceph_insights = RegistryPoint()
    ceph_log = RegistryPoint(multi_output=True, filterable=True)
    ceph_osd_df = RegistryPoint()
    ceph_osd_dump = RegistryPoint()
    ceph_osd_ec_profile_get = RegistryPoint(multi_output=True)
    ceph_osd_ec_profile_ls = RegistryPoint()
    ceph_osd_log = RegistryPoint(multi_output=True, filterable=True)
    ceph_osd_tree = RegistryPoint()
    ceph_osd_tree_text = RegistryPoint()
    ceph_report = RegistryPoint()
    ceph_s = RegistryPoint()
    ceph_v = RegistryPoint()
    certificates_enddate = RegistryPoint()
    cgroups = RegistryPoint()
    chkconfig = RegistryPoint()
    chrony_conf = RegistryPoint()
    chronyc_sources = RegistryPoint()
    cib_xml = RegistryPoint()
    cinder_api_log = RegistryPoint(filterable=True)
    cinder_conf = RegistryPoint()
    cinder_volume_log = RegistryPoint(filterable=True)
    cloud_init_custom_network = RegistryPoint()
    cloud_cfg = RegistryPoint(filterable=True)
    cloud_init_log = RegistryPoint(filterable=True)
    cluster_conf = RegistryPoint(filterable=True)
    cmdline = RegistryPoint()
    cni_podman_bridge_conf = RegistryPoint()
    cobbler_modules_conf = RegistryPoint()
    cobbler_settings = RegistryPoint()
    corosync = RegistryPoint()
    corosync_cmapctl = RegistryPoint(multi_output=True)
    corosync_conf = RegistryPoint()
    cpe = RegistryPoint()
    cpu_cores = RegistryPoint(multi_output=True)
    cpu_siblings = RegistryPoint(multi_output=True)
    cpu_smt_active = RegistryPoint()
    cpu_smt_control = RegistryPoint()
    cpu_vulns = RegistryPoint(multi_output=True)
    cpu_vulns_meltdown = RegistryPoint()
    cpu_vulns_spectre_v1 = RegistryPoint()
    cpu_vulns_spectre_v2 = RegistryPoint()
    cpu_vulns_spec_store_bypass = RegistryPoint()
    cpuinfo_max_freq = RegistryPoint()
    cpuinfo = RegistryPoint()
    cpupower_frequency_info = RegistryPoint()
    cpuset_cpus = RegistryPoint()
    cron_daily_rhsmd = RegistryPoint(filterable=True)
    crypto_policies_config = RegistryPoint()
    crypto_policies_state_current = RegistryPoint()
    crypto_policies_opensshserver = RegistryPoint()
    crypto_policies_bind = RegistryPoint()
    crt = RegistryPoint()
    cups_ppd = RegistryPoint(multi_output=True)
    current_clocksource = RegistryPoint()
    date_iso = RegistryPoint()
    date = RegistryPoint()
    date_utc = RegistryPoint()
    designate_conf = RegistryPoint(filterable=True)
    dcbtool_gc_dcb = RegistryPoint(multi_output=True)
    df__alP = RegistryPoint()
    df__al = RegistryPoint()
    df__li = RegistryPoint()
    dig_dnssec = RegistryPoint()
    dig_edns = RegistryPoint()
    dig_noedns = RegistryPoint()
    dig = RegistryPoint()
    dirsrv = RegistryPoint()
    dirsrv_access = RegistryPoint(multi_output=True, filterable=True)
    dirsrv_errors = RegistryPoint(multi_output=True, filterable=True)
    display_java = RegistryPoint()
    display_name = RegistryPoint()
    dm_mod_use_blk_mq = RegistryPoint()
    dmesg = RegistryPoint(filterable=True)
    dmesg_log = RegistryPoint(filterable=True)
    dmidecode = RegistryPoint()
    dmsetup_info = RegistryPoint()
    dmsetup_status = RegistryPoint()
    dnf_conf = RegistryPoint(filterable=True)
    dnf_modules = RegistryPoint()
    dnf_module_list = RegistryPoint()
    dnf_module_info = RegistryPoint()
    dnsmasq_config = RegistryPoint(multi_output=True)
    docker_container_inspect = RegistryPoint(multi_output=True)
    docker_host_machine_id = RegistryPoint()
    docker_image_inspect = RegistryPoint(multi_output=True)
    docker_info = RegistryPoint()
    docker_list_containers = RegistryPoint()
    docker_list_images = RegistryPoint()
    docker_network = RegistryPoint()
    docker_storage = RegistryPoint()
    docker_storage_setup = RegistryPoint()
    docker_sysconfig = RegistryPoint()
    dotnet_version = RegistryPoint()
    doveconf = RegistryPoint(filterable=True)
    dracut_kdump_capture_service = RegistryPoint()
    dse_ldif = RegistryPoint(multi_output=True, filterable=True)
    du_dirs = RegistryPoint(multi_output=True)
    dumpe2fs_h = RegistryPoint(multi_output=True)
    engine_config_all = RegistryPoint()
    engine_db_query_vdsm_version = RegistryPoint()
    engine_log = RegistryPoint(filterable=True)
    etc_journald_conf_d = RegistryPoint(multi_output=True)
    etc_journald_conf = RegistryPoint()
    etc_machine_id = RegistryPoint()
    etc_udev_40_redhat_rules = RegistryPoint(filterable=True)
    etc_udev_oracle_asm_rules = RegistryPoint(multi_output=True, filterable=True)
    etcd_conf = RegistryPoint(filterable=True)
    ethernet_interfaces = RegistryPoint()
    ethtool_a = RegistryPoint(multi_output=True)
    ethtool_c = RegistryPoint(multi_output=True)
    ethtool_g = RegistryPoint(multi_output=True)
    ethtool_i = RegistryPoint(multi_output=True)
    ethtool_k = RegistryPoint(multi_output=True)
    ethtool = RegistryPoint(multi_output=True)
    ethtool_S = RegistryPoint(multi_output=True)
    ethtool_T = RegistryPoint(multi_output=True)
    exim_conf = RegistryPoint()
    facter = RegistryPoint()
    fc_match = RegistryPoint()
    fcoeadm_i = RegistryPoint()
    fdisk_l = RegistryPoint()
    fdisk_l_sos = RegistryPoint(multi_output=True)
    findmnt_lo_propagation = RegistryPoint()
    firewall_cmd_list_all_zones = RegistryPoint()
    firewalld_conf = RegistryPoint(filterable=True)
    foreman_production_log = RegistryPoint(filterable=True)
    foreman_proxy_conf = RegistryPoint()
    foreman_proxy_log = RegistryPoint(filterable=True)
    foreman_satellite_log = RegistryPoint(filterable=True)
    foreman_ssl_access_ssl_log = RegistryPoint(filterable=True)
    foreman_ssl_error_ssl_log = RegistryPoint(filterable=True)
    foreman_rake_db_migrate_status = RegistryPoint()
    foreman_tasks_config = RegistryPoint(filterable=True)
    freeipa_healthcheck_log = RegistryPoint()
    fstab = RegistryPoint()
    fw_devices = RegistryPoint()
    fw_security = RegistryPoint()
    galera_cnf = RegistryPoint()
    gcp_instance_type = RegistryPoint()
    gcp_license_codes = RegistryPoint()
    getcert_list = RegistryPoint()
    getconf_page_size = RegistryPoint()
    getenforce = RegistryPoint()
    getsebool = RegistryPoint()
    gfs2_file_system_block_size = RegistryPoint(multi_output=True)
    glance_api_conf = RegistryPoint()
    glance_api_log = RegistryPoint(filterable=True)
    glance_cache_conf = RegistryPoint()
    glance_registry_conf = RegistryPoint()
    gluster_v_info = RegistryPoint()
    gluster_v_status = RegistryPoint()
    gluster_peer_status = RegistryPoint()
    gnocchi_conf = RegistryPoint(filterable=True)
    gnocchi_metricd_log = RegistryPoint(filterable=True)
    greenboot_status = RegistryPoint(filterable=True)
    grubenv = RegistryPoint()
    grub_conf = RegistryPoint()
    grub_config_perms = RegistryPoint()
    grub_efi_conf = RegistryPoint()
    grub1_config_perms = RegistryPoint()
    grub2_cfg = RegistryPoint()
    grub2_efi_cfg = RegistryPoint()
    grubby_default_index = RegistryPoint()
    grubby_default_kernel = RegistryPoint()
    hammer_ping = RegistryPoint()
    hammer_task_list = RegistryPoint()
    satellite_enabled_features = RegistryPoint()
    haproxy_cfg = RegistryPoint()
    haproxy_cfg_scl = RegistryPoint()
    heat_api_log = RegistryPoint(filterable=True)
    heat_conf = RegistryPoint()
    heat_crontab = RegistryPoint()
    heat_crontab_container = RegistryPoint()
    heat_engine_log = RegistryPoint(filterable=True)
    hostname = RegistryPoint()
    hostname_default = RegistryPoint()
    hostname_short = RegistryPoint()
    hosts = RegistryPoint()
    hponcfg_g = RegistryPoint()
    httpd_access_log = RegistryPoint(filterable=True)
    httpd_cert_info_in_nss = RegistryPoint(multi_output=True, filterable=True)
    httpd_conf = RegistryPoint(multi_output=True)
    httpd_conf_scl_httpd24 = RegistryPoint(multi_output=True)
    httpd_conf_scl_jbcs_httpd24 = RegistryPoint(multi_output=True)
    httpd_error_log = RegistryPoint(filterable=True)
    httpd24_httpd_error_log = RegistryPoint(filterable=True)
    jbcs_httpd24_httpd_error_log = RegistryPoint(filterable=True)
    httpd_limits = RegistryPoint(multi_output=True)
    httpd_M = RegistryPoint(multi_output=True)
    httpd_on_nfs = RegistryPoint()
    httpd_ssl_access_log = RegistryPoint(filterable=True)
    httpd_ssl_cert_enddate = RegistryPoint(multi_output=True)
    httpd_ssl_error_log = RegistryPoint(filterable=True)
    httpd_V = RegistryPoint(multi_output=True)
    virt_uuid_facts = RegistryPoint()
    ifcfg = RegistryPoint(multi_output=True)
    ifcfg_static_route = RegistryPoint(multi_output=True)
    ifconfig = RegistryPoint()
    imagemagick_policy = RegistryPoint(multi_output=True, filterable=True)
    initctl_lst = RegistryPoint()
    init_ora = RegistryPoint()
    initscript = RegistryPoint(multi_output=True)
    init_process_cgroup = RegistryPoint()
    insights_client_conf = RegistryPoint(filterable=True)
    installed_rpms = RegistryPoint()
    interrupts = RegistryPoint()
    ip6tables_permanent = RegistryPoint()
    ip6tables = RegistryPoint()
    ip_addr = RegistryPoint()
    ip_addresses = RegistryPoint()
    ipaupgrade_log = RegistryPoint(filterable=True)
    ipcs_m = RegistryPoint()
    ipcs_m_p = RegistryPoint()
    ipcs_s = RegistryPoint()
    ipcs_s_i = RegistryPoint(multi_output=True)
    ip_netns_exec_namespace_lsof = RegistryPoint(multi_output=True, filterable=True)
    ip_route_show_table_all = RegistryPoint()
    ip_s_link = RegistryPoint()
    ipsec_conf = RegistryPoint(filterable=True)
    iptables_permanent = RegistryPoint()
    iptables = RegistryPoint()
    ipv4_neigh = RegistryPoint()
    ipv6_neigh = RegistryPoint()
    ip_neigh_show = RegistryPoint()
    ironic_conf = RegistryPoint(filterable=True)
    ironic_inspector_log = RegistryPoint(filterable=True)
    iscsiadm_m_session = RegistryPoint()
    jboss_domain_server_log = RegistryPoint(multi_output=True, filterable=True)
    jboss_standalone_server_log = RegistryPoint(multi_output=True, filterable=True)
    jboss_standalone_main_config = RegistryPoint(multi_output=True)
    jboss_version = RegistryPoint(multi_output=True)
    journal_all = RegistryPoint(filterable=True)
    journal_since_boot = RegistryPoint(filterable=True)
    katello_service_status = RegistryPoint(filterable=True)
    kdump_conf = RegistryPoint()
    kerberos_kdc_log = RegistryPoint(filterable=True)
    kernel_config = RegistryPoint(multi_output=True, filterable=True)
    kexec_crash_loaded = RegistryPoint()
    kexec_crash_size = RegistryPoint()
    keystone_conf = RegistryPoint()
    keystone_crontab = RegistryPoint()
    keystone_crontab_container = RegistryPoint()
    keystone_log = RegistryPoint(filterable=True)
    kpatch_list = RegistryPoint()
    krb5 = RegistryPoint(multi_output=True)
    ksmstate = RegistryPoint()
    kubepods_cpu_quota = RegistryPoint(multi_output=True)
    lastupload = RegistryPoint(multi_output=True)
    ld_library_path_of_user = RegistryPoint()
    ldif_config = RegistryPoint(multi_output=True)
    libssh_client_config = RegistryPoint(filterable=True)
    libssh_server_config = RegistryPoint(filterable=True)
    libvirtd_log = RegistryPoint(filterable=True)
    libvirtd_qemu_log = RegistryPoint(multi_output=True, filterable=True)
    limits_conf = RegistryPoint(multi_output=True)
    locale = RegistryPoint()
    localtime = RegistryPoint()
    logrotate_conf = RegistryPoint(multi_output=True)
    lpfc_max_luns = RegistryPoint()
    lpstat_p = RegistryPoint()
    lpstat_protocol_printers = RegistryPoint()
    ls_boot = RegistryPoint()
    ls_dev = RegistryPoint()
    ls_disk = RegistryPoint()
    ls_docker_volumes = RegistryPoint()
    ls_edac_mc = RegistryPoint()
    ls_etc = RegistryPoint()
    ls_ipa_idoverride_memberof = RegistryPoint()
    ls_lib_firmware = RegistryPoint()
    ls_ocp_cni_openshift_sdn = RegistryPoint()
    ls_origin_local_volumes_pods = RegistryPoint()
    ls_osroot = RegistryPoint()
    ls_run_systemd_generator = RegistryPoint()
    ls_R_var_lib_nova_instances = RegistryPoint()
    ls_sys_firmware = RegistryPoint()
    ls_usr_bin = RegistryPoint(filterable=True)
    ls_usr_lib64 = RegistryPoint(filterable=True)
    ls_usr_sbin = RegistryPoint(filterable=True)
    ls_var_cache_pulp = RegistryPoint()
    ls_var_lib_mongodb = RegistryPoint()
    ls_var_lib_nova_instances = RegistryPoint()
    ls_var_log = RegistryPoint()
    ls_var_opt_mssql = RegistryPoint()
    ls_var_opt_mssql_log = RegistryPoint()
    ls_var_run = RegistryPoint()
    ls_var_spool_clientmq = RegistryPoint()
    ls_var_spool_postfix_maildrop = RegistryPoint()
    ls_var_tmp = RegistryPoint(filterable=True)
    ls_var_www = RegistryPoint()
    ls_tmp = RegistryPoint(filterable=True)
    lsblk = RegistryPoint()
    lsblk_pairs = RegistryPoint()
    lscpu = RegistryPoint()
    lsinitrd = RegistryPoint(filterable=True)
    lsinitrd_lvm_conf = RegistryPoint()
    lsmod = RegistryPoint()
    lsof = RegistryPoint(filterable=True)
    lspci = RegistryPoint()
    lspci_vmmkn = RegistryPoint()
    lssap = RegistryPoint()
    lsscsi = RegistryPoint()
    lsvmbus = RegistryPoint()
    lvdisplay = RegistryPoint()
    lvm_conf = RegistryPoint(filterable=True)
    lvmconfig = RegistryPoint()
    lvs_noheadings = RegistryPoint()
    lvs_noheadings_all = RegistryPoint()
    lvs = RegistryPoint()
    mac_addresses = RegistryPoint(multi_output=True)
    machine_id = RegistryPoint()
    manila_conf = RegistryPoint()
    mariadb_log = RegistryPoint(filterable=True)
    max_uid = RegistryPoint()
    mdadm_E = RegistryPoint(multi_output=True)
    md5chk_files = RegistryPoint(multi_output=True)
    mdstat = RegistryPoint()
    meminfo = RegistryPoint()
    messages = RegistryPoint(filterable=True)
    metadata_json = RegistryPoint(raw=True)
    satellite_missed_pulp_agent_queues = RegistryPoint()
    mistral_executor_log = RegistryPoint(filterable=True)
    mlx4_port = RegistryPoint(multi_output=True)
    modinfo_i40e = RegistryPoint()
    modinfo_igb = RegistryPoint()
    modinfo_ixgbe = RegistryPoint()
    modinfo_veth = RegistryPoint()
    modinfo_vmxnet3 = RegistryPoint()
    modinfo = RegistryPoint(multi_output=True)
    modinfo_all = RegistryPoint()
    modprobe = RegistryPoint(multi_output=True)
    mokutil_sbstate = RegistryPoint()
    mongod_conf = RegistryPoint(multi_output=True, filterable=True)
    mount = RegistryPoint()
    mounts = RegistryPoint()
    mssql_conf = RegistryPoint()
    mssql_api_assessment = RegistryPoint()
    mssql_tls_cert_enddate = RegistryPoint()
    multicast_querier = RegistryPoint()
    multipath_conf = RegistryPoint()
    multipath_conf_initramfs = RegistryPoint()
    multipath__v4__ll = RegistryPoint()
    mysqladmin_status = RegistryPoint()
    mysqladmin_vars = RegistryPoint()
    mysql_log = RegistryPoint(multi_output=True, filterable=True)
    mysqld_limits = RegistryPoint()
    named_checkconf_p = RegistryPoint(filterable=True)
    named_conf = RegistryPoint(filterable=True)
    namespace = RegistryPoint()
    ndctl_list_Ni = RegistryPoint()
    netconsole = RegistryPoint()
    netstat_agn = RegistryPoint()
    netstat_i = RegistryPoint()
    netstat = RegistryPoint()
    netstat_s = RegistryPoint()
    networkmanager_conf = RegistryPoint()
    networkmanager_dispatcher_d = RegistryPoint(multi_output=True)
    neutron_conf = RegistryPoint(filterable=True)
    neutron_sriov_agent = RegistryPoint(filterable=True)
    neutron_dhcp_agent_ini = RegistryPoint(filterable=True)
    neutron_l3_agent_ini = RegistryPoint(filterable=True)
    neutron_l3_agent_log = RegistryPoint(filterable=True)
    neutron_metadata_agent_ini = RegistryPoint(filterable=True)
    neutron_metadata_agent_log = RegistryPoint(filterable=True)
    neutron_ml2_conf = RegistryPoint(filterable=True)
    neutron_ovs_agent_log = RegistryPoint(filterable=True)
    neutron_plugin_ini = RegistryPoint()
    neutron_server_log = RegistryPoint(filterable=True)
    nfnetlink_queue = RegistryPoint()
    nfs_exports_d = RegistryPoint(multi_output=True)
    nfs_exports = RegistryPoint()
    nginx_conf = RegistryPoint(multi_output=True)
    nginx_ssl_cert_enddate = RegistryPoint(multi_output=True)
    nmcli_conn_show = RegistryPoint()
    nmcli_dev_show = RegistryPoint()
    nmcli_dev_show_sos = RegistryPoint(multi_output=True)
    nova_api_log = RegistryPoint(filterable=True)
    nova_compute_log = RegistryPoint(filterable=True)
    nova_conf = RegistryPoint()
    nova_crontab = RegistryPoint()
    nova_crontab_container = RegistryPoint()
    nova_uid = RegistryPoint()
    nova_migration_uid = RegistryPoint()
    nscd_conf = RegistryPoint(filterable=True)
    nss_rhel7 = RegistryPoint()
    nsswitch_conf = RegistryPoint(filterable=True)
    ntp_conf = RegistryPoint()
    ntpq_leap = RegistryPoint()
    ntpq_pn = RegistryPoint()
    ntptime = RegistryPoint()
    numa_cpus = RegistryPoint(multi_output=True)
    numeric_user_group_name = RegistryPoint()
    nvme_core_io_timeout = RegistryPoint()
    oc_get_bc = RegistryPoint()
    oc_get_build = RegistryPoint()
    oc_get_clusterrole_with_config = RegistryPoint()
    oc_get_clusterrolebinding_with_config = RegistryPoint()
    oc_get_dc = RegistryPoint()
    oc_get_egressnetworkpolicy = RegistryPoint()
    oc_get_endpoints = RegistryPoint()
    oc_get_event = RegistryPoint()
    oc_get_node = RegistryPoint()
    oc_get_pod = RegistryPoint()
    oc_get_project = RegistryPoint()
    oc_get_pvc = RegistryPoint()
    oc_get_pv = RegistryPoint()
    oc_get_rc = RegistryPoint()
    oc_get_rolebinding = RegistryPoint()
    oc_get_role = RegistryPoint()
    oc_get_route = RegistryPoint()
    oc_get_service = RegistryPoint()
    oc_get_configmap = RegistryPoint()
    octavia_conf = RegistryPoint(filterable=True)
    odbc_ini = RegistryPoint(filterable=True)
    odbcinst_ini = RegistryPoint()
    open_vm_tools_stat_raw_text_session = RegistryPoint()
    openvswitch_other_config = RegistryPoint()
    openvswitch_server_log = RegistryPoint(filterable=True)
    openshift_certificates = RegistryPoint(multi_output=True)
    openshift_fluentd_environ = RegistryPoint(multi_output=True)
    openshift_hosts = RegistryPoint(filterable=True)
    openshift_router_environ = RegistryPoint(multi_output=True)
    openvswitch_daemon_log = RegistryPoint(filterable=True)
    openvswitch_server_log = RegistryPoint(filterable=True)
    osa_dispatcher_log = RegistryPoint(filterable=True)
    ose_master_config = RegistryPoint()
    ose_node_config = RegistryPoint()
    os_release = RegistryPoint()
    ovirt_engine_boot_log = RegistryPoint(filterable=True)
    ovirt_engine_confd = RegistryPoint(multi_output=True)
    ovirt_engine_console_log = RegistryPoint(filterable=True)
    ovirt_engine_server_log = RegistryPoint(filterable=True)
    ovirt_engine_ui_log = RegistryPoint(filterable=True)
    ovs_appctl_fdb_show_bridge = RegistryPoint(multi_output=True)
    ovs_ofctl_dump_flows = RegistryPoint(multi_output=True)
    ovs_vsctl_list_bridge = RegistryPoint()
    ovs_vsctl_show = RegistryPoint()
    ovs_vswitchd_limits = RegistryPoint()
    pacemaker_log = RegistryPoint(filterable=True)
    package_provides_command = RegistryPoint(filterable=True)
    package_provides_java = RegistryPoint(multi_output=True)
    package_provides_httpd = RegistryPoint(multi_output=True)
    pam_conf = RegistryPoint()
    parted__l = RegistryPoint()
    partitions = RegistryPoint()
    passenger_status = RegistryPoint()
    password_auth = RegistryPoint()
    pci_rport_target_disk_paths = RegistryPoint()
    pcp_metrics = RegistryPoint()
    pcs_config = RegistryPoint()
    pcs_quorum_status = RegistryPoint()
    pcs_status = RegistryPoint()
    php_ini = RegistryPoint(filterable=True)
    pluginconf_d = RegistryPoint(multi_output=True)
    pmlog_summary = RegistryPoint()
    pmrep_metrics = RegistryPoint()
    podman_container_inspect = RegistryPoint(multi_output=True)
    podman_image_inspect = RegistryPoint(multi_output=True)
    podman_list_containers = RegistryPoint()
    podman_list_images = RegistryPoint()
    postconf_builtin = RegistryPoint(filterable=True)
    postconf = RegistryPoint(filterable=True)
    postgresql_conf = RegistryPoint()
    postgresql_log = RegistryPoint(multi_output=True, filterable=True)
    prev_uploader_log = RegistryPoint()
    proc_netstat = RegistryPoint()
    proc_slabinfo = RegistryPoint()
    proc_snmp_ipv4 = RegistryPoint()
    proc_snmp_ipv6 = RegistryPoint()
    proc_stat = RegistryPoint()
    ps_alxwww = RegistryPoint(filterable=True)
    ps_aux = RegistryPoint(filterable=True)
    ps_auxcww = RegistryPoint()
    ps_auxww = RegistryPoint(filterable=True)
    ps_ef = RegistryPoint(filterable=True)
    ps_eo = RegistryPoint()
    ps_eo_cmd = RegistryPoint()
    pulp_worker_defaults = RegistryPoint()
    puppet_ssl_cert_ca_pem = RegistryPoint()
    puppetserver_config = RegistryPoint(filterable=True)
    puppet_ca_cert_expire_date = RegistryPoint()
    pvs_noheadings = RegistryPoint()
    pvs_noheadings_all = RegistryPoint()
    pvs = RegistryPoint()
    qemu_conf = RegistryPoint()
    qemu_xml = RegistryPoint(multi_output=True)
    ql2xmaxlun = RegistryPoint()
    qpid_stat_g = RegistryPoint()
    qpid_stat_q = RegistryPoint()
    qpid_stat_u = RegistryPoint()
    qpidd_conf = RegistryPoint()
    rabbitmq_env = RegistryPoint()
    rabbitmq_logs = RegistryPoint(multi_output=True, filterable=True)
    rabbitmq_policies = RegistryPoint()
    rabbitmq_queues = RegistryPoint()
    rabbitmq_report = RegistryPoint()
    rabbitmq_report_of_containers = RegistryPoint(multi_output=True)
    rabbitmq_startup_err = RegistryPoint(filterable=True)
    rabbitmq_startup_log = RegistryPoint(filterable=True)
    rabbitmq_users = RegistryPoint()
    rc_local = RegistryPoint()
    rdma_conf = RegistryPoint()
    readlink_e_etc_mtab = RegistryPoint()
    readlink_e_shift_cert_client = RegistryPoint()
    readlink_e_shift_cert_server = RegistryPoint()
    redhat_release = RegistryPoint()
    resolv_conf = RegistryPoint()
    rhev_data_center = RegistryPoint()
    rhv_log_collector_analyzer = RegistryPoint()
    rhn_charsets = RegistryPoint()
    rhn_conf = RegistryPoint()
    rhn_entitlement_cert_xml = RegistryPoint(multi_output=True)
    rhn_hibernate_conf = RegistryPoint()
    rhn_schema_stats = RegistryPoint()
    rhn_schema_version = RegistryPoint()
    rhn_search_daemon_log = RegistryPoint(filterable=True)
    rhn_server_satellite_log = RegistryPoint(filterable=True)
    rhn_server_xmlrpc_log = RegistryPoint(filterable=True)
    rhn_taskomatic_daemon_log = RegistryPoint(filterable=False)
    rhosp_release = RegistryPoint()
    rhsm_conf = RegistryPoint()
    rhsm_katello_default_ca_cert = RegistryPoint()
    rhsm_log = RegistryPoint(filterable=True)
    rhsm_releasever = RegistryPoint()
    rndc_status = RegistryPoint()
    root_crontab = RegistryPoint()
    ros_config = RegistryPoint()
    route = RegistryPoint()
    rpm_ostree_status = RegistryPoint()
    rpm_V_packages = RegistryPoint()
    rsyslog_conf = RegistryPoint(filterable=True, multi_output=True)
    samba = RegistryPoint(filterable=True)
    sap_dev_disp = RegistryPoint(multi_output=True, filterable=True)
    sap_dev_rd = RegistryPoint(multi_output=True, filterable=True)
    sap_hana_landscape = RegistryPoint(multi_output=True)
    sap_hdb_version = RegistryPoint(multi_output=True)
    sap_host_profile = RegistryPoint(filterable=True)
    sapcontrol_getsystemupdatelist = RegistryPoint()
    saphostctl_getcimobject_sapinstance = RegistryPoint(filterable=True)
    saphostexec_status = RegistryPoint()
    saphostexec_version = RegistryPoint()
    sat5_insights_properties = RegistryPoint()
    satellite_compute_resources = RegistryPoint()
    satellite_content_hosts_count = RegistryPoint()
    satellite_core_taskreservedresource_count = RegistryPoint()
    satellite_custom_ca_chain = RegistryPoint()
    satellite_custom_hiera = RegistryPoint()
    satellite_katello_empty_url_repositories = RegistryPoint()
    satellite_mongodb_storage_engine = RegistryPoint()
    satellite_non_yum_type_repos = RegistryPoint()
    satellite_sca_status = RegistryPoint()
    satellite_settings = RegistryPoint()
    satellite_version_rb = RegistryPoint()
    satellite_yaml = RegistryPoint()
    scheduler = RegistryPoint(multi_output=True)
    sched_rt_runtime_us = RegistryPoint()
    scsi = RegistryPoint()
    scsi_mod_max_report_luns = RegistryPoint()
    scsi_mod_use_blk_mq = RegistryPoint()
    sctp_asc = RegistryPoint()
    sctp_eps = RegistryPoint()
    sctp_snmp = RegistryPoint()
    scsi_eh_deadline = RegistryPoint(multi_output=True)
    scsi_fwver = RegistryPoint(multi_output=True)
    sealert = RegistryPoint()
    secure = RegistryPoint(filterable=True)
    selinux_config = RegistryPoint()
    sestatus = RegistryPoint()
    setup_named_chroot = RegistryPoint(filterable=True)
    smartctl = RegistryPoint(multi_output=True)
    smartpdc_settings = RegistryPoint(filterable=True)
    smbstatus_p = RegistryPoint()
    smbstatus_S = RegistryPoint()
    sockstat = RegistryPoint()
    softnet_stat = RegistryPoint()
    software_collections_list = RegistryPoint()
    spamassassin_channels = RegistryPoint()
    spfile_ora = RegistryPoint(multi_output=True)
    ssh_config_d = RegistryPoint(multi_output=True, filterable=True)
    ssh_config = RegistryPoint(filterable=True)
    ssh_foreman_config = RegistryPoint(filterable=True)
    ssh_foreman_proxy_config = RegistryPoint(filterable=True)
    sshd_config_perms = RegistryPoint()
    sshd_config = RegistryPoint(filterable=True)
    ss = RegistryPoint()
    sssd_config = RegistryPoint()
    sssd_logs = RegistryPoint(multi_output=True, filterable=True)
    samba_logs = RegistryPoint(multi_output=True, filterable=True)
    subscription_manager_id = RegistryPoint()
    subscription_manager_list_consumed = RegistryPoint()
    subscription_manager_list_installed = RegistryPoint()
    subscription_manager_installed_product_ids = RegistryPoint(filterable=True)
    subscription_manager_release_show = RegistryPoint()
    swift_conf = RegistryPoint()
    swift_log = RegistryPoint(filterable=True)
    swift_object_expirer_conf = RegistryPoint()
    swift_proxy_server_conf = RegistryPoint()
    sys_kernel_sched_features = RegistryPoint()
    sysconfig_chronyd = RegistryPoint()
    sysconfig_grub = RegistryPoint()
    sysconfig_httpd = RegistryPoint()
    sysconfig_irqbalance = RegistryPoint()
    sysconfig_kdump = RegistryPoint()
    sysconfig_libvirt_guests = RegistryPoint()
    sysconfig_memcached = RegistryPoint()
    sysconfig_mongod = RegistryPoint(multi_output=True)
    sysconfig_network = RegistryPoint()
    sysconfig_ntpd = RegistryPoint()
    sysconfig_oracleasm = RegistryPoint()
    sysconfig_prelink = RegistryPoint()
    sysconfig_sshd = RegistryPoint()
    sysconfig_virt_who = RegistryPoint()
    sysctl_conf_initramfs = RegistryPoint(multi_output=True)
    sysctl_conf = RegistryPoint()
    sysctl = RegistryPoint()
    systemctl_cat_dnsmasq_service = RegistryPoint()
    systemctl_cat_rpcbind_socket = RegistryPoint()
    systemctl_cinder_volume = RegistryPoint()
    systemctl_httpd = RegistryPoint()
    systemctl_nginx = RegistryPoint()
    systemctl_list_unit_files = RegistryPoint()
    systemctl_list_units = RegistryPoint()
    systemctl_mariadb = RegistryPoint()
    systemctl_pulp_workers = RegistryPoint()
    systemctl_pulp_resmg = RegistryPoint()
    systemctl_pulp_celerybeat = RegistryPoint()
    systemctl_qpidd = RegistryPoint()
    systemctl_qdrouterd = RegistryPoint()
    systemctl_show_all_services = RegistryPoint()
    systemctl_show_target = RegistryPoint()
    systemctl_smartpdc = RegistryPoint()
    systemctl_status_all = RegistryPoint(filterable=True)
    systemd_analyze_blame = RegistryPoint()
    systemd_docker = RegistryPoint()
    systemd_logind_conf = RegistryPoint()
    systemd_openshift_node = RegistryPoint()
    systemd_system_conf = RegistryPoint()
    systemd_system_origin_accounting = RegistryPoint()
    systemid = RegistryPoint()
    systool_b_scsi_v = RegistryPoint()
    sys_vmbus_device_id = RegistryPoint(multi_output=True)
    sys_vmbus_class_id = RegistryPoint(multi_output=True)
    testparm_s = RegistryPoint(filterable=True)
    testparm_v_s = RegistryPoint(filterable=True)
    tags = RegistryPoint()
    teamdctl_config_dump = RegistryPoint(multi_output=True)
    teamdctl_state_dump = RegistryPoint(multi_output=True)
    thp_enabled = RegistryPoint()
    thp_use_zero_page = RegistryPoint()
    tmpfilesd = RegistryPoint(multi_output=True)
    tomcat_server_xml = RegistryPoint(multi_output=True)
    tomcat_vdc_fallback = RegistryPoint()
    tomcat_vdc_targeted = RegistryPoint(multi_output=True)
    tomcat_web_xml = RegistryPoint(multi_output=True)
    tuned_adm = RegistryPoint()
    tuned_conf = RegistryPoint()
    udev_persistent_net_rules = RegistryPoint()
    udev_fc_wwpn_id_rules = RegistryPoint(filterable=True)
    uname = RegistryPoint()
    up2date = RegistryPoint()
    up2date_log = RegistryPoint(filterable=True)
    uploader_log = RegistryPoint()
    uptime = RegistryPoint()
    usr_journald_conf_d = RegistryPoint(multi_output=True)
    var_qemu_xml = RegistryPoint(multi_output=True)
    vdsm_conf = RegistryPoint()
    vdsm_id = RegistryPoint()
    vdsm_import_log = RegistryPoint(multi_output=True, filterable=True)
    vdsm_log = RegistryPoint(filterable=True)
    vdsm_logger_conf = RegistryPoint()
    version_info = RegistryPoint()
    vdo_status = RegistryPoint()
    vgdisplay = RegistryPoint()
    vgs_noheadings = RegistryPoint()
    vgs_noheadings_all = RegistryPoint()
    vgs = RegistryPoint()
    vhost_net_zero_copy_tx = RegistryPoint()
    virsh_list_all = RegistryPoint()
    virt_what = RegistryPoint()
    virt_who_conf = RegistryPoint(multi_output=True, filterable=True)
    virtlogd_conf = RegistryPoint(filterable=True)
    vma_ra_enabled = RegistryPoint()
    vmcore_dmesg = RegistryPoint(multi_output=True, filterable=True)
    vmware_tools_conf = RegistryPoint()
    vsftpd_conf = RegistryPoint(filterable=True)
    vsftpd = RegistryPoint()
    x86_pti_enabled = RegistryPoint()
    x86_ibpb_enabled = RegistryPoint()
    x86_ibrs_enabled = RegistryPoint()
    x86_retp_enabled = RegistryPoint()
    xfs_info = RegistryPoint(multi_output=True)
    xinetd_conf = RegistryPoint(multi_output=True)
    yum_conf = RegistryPoint()
    yum_list_available = RegistryPoint()
    yum_list_installed = RegistryPoint()
    yum_log = RegistryPoint()
    yum_repolist = RegistryPoint()
    yum_repos_d = RegistryPoint(multi_output=True)
    yum_updateinfo = RegistryPoint()
    yum_updates = RegistryPoint()
    zdump_v = RegistryPoint()
    zipl_conf = RegistryPoint()
    sendq_socket_buffer = RegistryPoint()
    recvq_socket_buffer = RegistryPoint()



from collections import namedtuple
from insights.combiners.ipv6 import IPv6
from insights.parsers.cmdline import CmdLine
from insights.parsers.lsmod import LsMod
from insights.parsers.modprobe import ModProbe
from insights.parsers.sysctl import Sysctl
from insights.parsers.uname import Uname
from insights.tests import context_wrap

Case = namedtuple('Case', ['cmdline', 'lsmod', 'modprobe', 'sysctl'])

UNAME_RHEL7 = '''
Linux localhost.localdomain 3.10.0-514.6.1.el7.x86_64 #1 SMP Sat Dec 10 11:15:38 EST 2016 x86_64 x86_64 x86_64 GNU/Linux
'''  # noqa

UNAME_RHEL6 = '''
Linux localhost.localdomain 2.6.32-642.el6.x86_64 #1 SMP Wed Apr 13 00:51:26 EDT 2016 x86_64 x86_64 x86_64 GNU/Linux
'''  # noqa

CMDLINE_DISABLED = '''
BOOT_IMAGE=/vmlinuz-3.10.0-514.6.1.el7.x86_64 root=/dev/mapper/rhel-root ro crashkernel=auto rd.lvm.lv=rhel/root rd.lvm.lv=rhel/swap ipv6.disable=1
'''  # noqa

CMDLINE_NOT_DISABLED = '''
BOOT_IMAGE=/vmlinuz-3.10.0-514.6.1.el7.x86_64 root=/dev/mapper/rhel-root ro crashkernel=auto rd.lvm.lv=rhel/root rd.lvm.lv=rhel/swap
'''  # noqa

CMDLINE_RHEL6_DISABLED = '''
ro root=/dev/mapper/VolGroup-lv_root rd_NO_LUKS LANG=en_US.UTF-8 rd_NO_MD rd_LVM_LV=VolGroup/lv_swap SYSFONT=latarcyrheb-sun16  rd_LVM_LV=VolGroup/lv_root  KEYBOARDTYPE=pc KEYTABLE=us rd_NO_DM rhgb quiet ipv6.disable=1
'''  # noqa

LSMOD_LOADED = '''
Module                  Size  Used by
vboxsf                 37955  1 
ipv6                  336282  12 
i2c_piix4              11232  0 
i2c_core               29132  1 i2c_piix4
snd_intel8x0           30524  0 
'''  # noqa

LSMOD_NOT_LOADED = '''
Module                  Size  Used by
vboxsf                 37955  1 
i2c_piix4              11232  0 
i2c_core               29132  1 i2c_piix4
snd_intel8x0           30524  0 
'''  # noqa

MODPROBE_NOT_DISABLED = '''
# framebuffer drivers
blacklist aty128fb
blacklist atyfb
blacklist radeonfb
blacklist i810fb
blacklist cirrusfb
blacklist intelfb
'''  # noqa

MODPROBE_DISABLED = '''
blacklist radeonfb
blacklist i810fb
blacklist cirrusfb
blacklist intelfb
options ipv6 disable=1
'''  # noqa

MODPROBE_FAKE = '''
blacklist radeonfb
blacklist i810fb
blacklist cirrusfb
blacklist intelfb
install ipv6 /bin/true
install blarfl /bin/true
'''  # noqa

MODPROBE_FAKE_COMMENTED = '''
blacklist radeonfb
blacklist i810fb
blacklist cirrusfb
blacklist intelfb
# install ipv6 /bin/true
'''  # noqa

SYSCTL_DISABLED = '''
net.ipv6.conf.all.disable_ipv6 = 1
'''  # noqa

SYSCTL_NOT_DISABLED = '''
net.ipv6.route.gc_elasticity = 9
net.ipv6.route.gc_interval = 30
net.ipv6.route.gc_min_interval = 0
net.ipv6.route.gc_min_interval_ms = 500
net.ipv6.route.gc_thresh = 1024
net.ipv6.route.gc_timeout = 60
'''  # noqa

CASES = [
    # noqa
    # RHEL7 not disabled
    (7, Case(CMDLINE_NOT_DISABLED, None, None, SYSCTL_NOT_DISABLED),
        (False, set())),
    # RHEL7 disabled via cmdline
    (7, Case(CMDLINE_DISABLED, None, None, SYSCTL_NOT_DISABLED),
        (True, set(['cmdline']))),
    # RHEL7 disabled via sysctl
    (7, Case(CMDLINE_NOT_DISABLED, None, None, SYSCTL_DISABLED),
        (True, set(['sysctl']))),
    # RHEL7 disabled by both cmdline and sysctl
    (7, Case(CMDLINE_DISABLED, None, None, SYSCTL_DISABLED),
        (True, set(['cmdline', 'sysctl']))),
    # RHEL7 with only uname provided
    (7, Case(None, None, None, None),
        (False, set())),
    # RHEL6 loaded not disabled
    (6, Case(None, LSMOD_LOADED, MODPROBE_NOT_DISABLED,
             SYSCTL_NOT_DISABLED), (False, set())),
    # RHEL6 not loaded but not disabled
    (6, Case(None, LSMOD_NOT_LOADED, MODPROBE_NOT_DISABLED,
             SYSCTL_NOT_DISABLED), (False, set())),
    # RHEL6 fake installed but loaded
    (6, Case(None, LSMOD_LOADED, MODPROBE_FAKE, SYSCTL_NOT_DISABLED),
        (False, set())),
    # RHEL6 not loaded, fake install commented
    (6, Case(None, LSMOD_NOT_LOADED, MODPROBE_FAKE_COMMENTED,
             SYSCTL_NOT_DISABLED), (False, set())),
    # RHEL6 loaded but disabled via modprobe
    (6, Case(None, LSMOD_LOADED, MODPROBE_DISABLED, SYSCTL_NOT_DISABLED),
        (True, set(['modprobe_disable']))),
    # RHEL6 not loaded, disabled via modprobe
    (6, Case(None, LSMOD_NOT_LOADED, MODPROBE_DISABLED,
             SYSCTL_NOT_DISABLED), (True, set(['modprobe_disable']))),
    # RHEL6 not loaded, disabled via fake install
    (6, Case(None, LSMOD_NOT_LOADED, MODPROBE_FAKE, SYSCTL_NOT_DISABLED),
        (True, set(['fake_install']))),
    # RHEL6 loaded but disabled by sysctl
    (6, Case(None, LSMOD_LOADED, MODPROBE_NOT_DISABLED, SYSCTL_DISABLED),
        (True, set(['sysctl']))),
    # RHEL6 not loaded, disabled by sysctl
    (6, Case(None, LSMOD_NOT_LOADED, MODPROBE_NOT_DISABLED,
             SYSCTL_DISABLED), (True, set(['sysctl']))),
    # RHEL6 disabled by modprobe and sysctl
    (6, Case(None, LSMOD_NOT_LOADED, MODPROBE_DISABLED, SYSCTL_DISABLED),
        (True, set(['sysctl', 'modprobe_disable']))),
    # RHEL6 with lsmod but no modprobe
    (6, Case(None, LSMOD_LOADED, None, None),
        (False, set())),
    # RHEL6 with modprobe but no lsmod
    (6, Case(None, None, MODPROBE_DISABLED, None),
        (True, set(['modprobe_disable']))),
    # RHEL6 with fake install but no lsmod (insufficient data)
    (6, Case(None, None, MODPROBE_FAKE, None),
        (False, set())),
    # RHEL6 with command line only
    (6, Case(CMDLINE_RHEL6_DISABLED, None, None, None),
        (True, set(['cmdline']))),
]


def test_integration():
    for rhel, case, result in CASES:
        context = dict()
        context[Uname] = Uname(context_wrap(
                               UNAME_RHEL7 if rhel == 7 else UNAME_RHEL6))

        if case.modprobe is not None:
            context[ModProbe] = ModProbe(context_wrap(case.modprobe,
                                         path='/etc/modprobe.d/ipv6.conf'))
        if case.lsmod is not None:
            context[LsMod] = LsMod(context_wrap(case.lsmod))

        if case.cmdline is not None:
            context[CmdLine] = CmdLine(context_wrap(case.cmdline))

        if case.sysctl is not None:
            context[Sysctl] = Sysctl(context_wrap(case.sysctl))

        un = context.get(Uname)
        mp = context.get(ModProbe)
        lsm = context.get(LsMod)
        cl = context.get(CmdLine)
        sct = context.get(Sysctl)

        ipv6 = IPv6(un, mp, lsm, cl, sct)
        assert ipv6.disabled() == result[0]
        assert ipv6.disabled_by() == result[1]



import doctest
from insights.tests import context_wrap
from insights.parsers import rsyslog_conf
from insights.parsers.rsyslog_conf import RsyslogConf


RSYSLOG_CONF = r"""
# Provides TCP syslog reception
:msg, regex, "\/vob\/.*\.[cpp|c|java]" /var/log/appMessages
*.info;mail.none;authpriv.none;cron.none                /var/log/messages
authpriv.*                                              /var/log/secure
$ModLoad imtcp
$InputTCPServerRun 10514
module(load="imuxsock"    # provides support for local system logging (e.g. via logger command)
       SysSock.Use="off") # Turn off message reception via local log socket;
module(load="builtin:omfile" Template="RSYSLOG_TraditionalFileFormat")
include(file="/etc/rsyslog.d/*.conf" mode="optional")
global(workDirectory="/var/lib/rsyslog")
*.info {
   action(
     type="omfile"
     name="hehe"
     file="/tmp/testnimei")
}
if $programname == 'prog1' then {
   action(type="omfile" file="/var/log/prog1.log")
   if $msg contains 'test' then
     action(type="omfile" file="/var/log/prog1test.log")
   else
     action(type="omfile" file="/var/log/prog1notest.log")
}
include(file="/etc/rsyslog.d/*.conf" mode="optional")
cron.*                                                  /var/log/cron
""".strip()


def test_rsyslog_conf_1():
    rsyslogconf = RsyslogConf(context_wrap(RSYSLOG_CONF))
    assert len(rsyslogconf) == 13
    assert rsyslogconf[2] == "authpriv.*                                              /var/log/secure"
    assert rsyslogconf[9] == """*.info { action( type="omfile" name="hehe" file="/tmp/testnimei") }"""


def test_rsyslog_doc_examples():
    env = {
        'rsysconf': RsyslogConf(context_wrap(RSYSLOG_CONF)),
    }
    failed, total = doctest.testmod(rsyslog_conf, globs=env)
    assert failed == 0



import os

from tempfile import mkdtemp
from insights import dr
from insights.core.plugins import component
from insights.core.serde import (serializer,
                                 deserializer,
                                 Hydration,
                                 marshal,
                                 unmarshal)
from insights.util import fs


class Foo(object):
    def __init__(self):
        self.a = 1
        self.b = 2


@component()
def thing():
    return Foo()


@serializer(Foo)
def serialize_foo(obj, root=None):
    return {"a": obj.a, "b": obj.b}


@deserializer(Foo)
def deserialize_foo(_type, data, root=None):
    foo = _type.__new__(_type)
    foo.a = data.get("a")
    foo.b = data.get("b")
    return foo


def test_marshal():
    foo = Foo()
    data = marshal(foo)
    assert data is not None
    d = data["object"]
    assert d["a"] == 1
    assert d["b"] == 2


def test_unmarshal():
    d = {
        "type": "insights.tests.test_serde.Foo",
        "object": {"a": 1, "b": 2}
    }
    foo = unmarshal(d)
    assert foo is not None
    assert foo.a == 1
    assert foo.b == 2


def test_hydrate_one():
    raw = {
        "name": dr.get_name(thing),
        "exec_time": 0.25,
        "ser_time": 0.05,
        "results": {
            "type": "insights.tests.test_serde.Foo",
            "object": {"a": 1, "b": 2}
        }
    }

    h = Hydration()
    key, result, exec_time, ser_time = h._hydrate_one(raw)
    assert key is thing
    assert isinstance(result, Foo)
    assert exec_time == 0.25
    assert ser_time == 0.05


def test_hydrate_one_multiple_results():
    raw = {
        "name": dr.get_name(thing),
        "exec_time": 0.5,
        "ser_time": 0.1,
        "results": [
            {
                "type": "insights.tests.test_serde.Foo",
                "object": {"a": 1, "b": 2}
            },
            {
                "type": "insights.tests.test_serde.Foo",
                "object": {"a": 3, "b": 4}
            },
        ]
    }

    h = Hydration()
    key, result, exec_time, ser_time = h._hydrate_one(raw)
    assert key is thing
    assert len(result) == 2
    assert exec_time == 0.5
    assert ser_time == 0.1
    assert result[0].a == 1
    assert result[0].b == 2
    assert result[1].a == 3
    assert result[1].b == 4


def test_round_trip():
    tmp_path = mkdtemp()
    try:
        h = Hydration(tmp_path)

        broker = dr.Broker()
        broker[thing] = Foo()
        broker.exec_times[thing] = 0.5
        h.dehydrate(thing, broker)
        fn = ".".join([dr.get_name(thing), h.ser_name])
        assert os.path.exists(os.path.join(h.meta_data, fn))

        broker = h.hydrate()
        assert thing in broker
        assert broker.exec_times[thing] >= 0.5
        foo = broker[thing]
        assert foo.a == 1
        assert foo.b == 2
    finally:
        pass
        if os.path.exists(tmp_path):
            fs.remove(tmp_path)



import doctest
import pytest
from insights.parsers import SkipException
from insights.parsers import rhev_data_center as ls_dc
from insights.tests import context_wrap


RHEV_DATA_CENTER_INSIGHTS = """
[{"name": "root", "group": "root", "path": "/rhev/data-center/mnt/r430-21.example.com:_home_nfsshare_data"}, {"name": "root", "group": "root", "path": "/rhev/data-center/mnt/r430-21.example.redhat.com:_home_nfsshare_iso"}, {"name": "root", "group": "root", "path": "/rhev/data-center/mnt/host1.example.com:_nfsshare_data/a384bf5d-db92-421e-926d-bfb99a6b4b28/images/b7d6cc07-d1f1-44b3-b3c0-7067ec7056a3/4d6e5dea-995f-4a4e-b487-0f70361f6137"}, {"name": "root", "group": "root", "path": "/rhev/data-center/mnt/host1.example.com:_nfsshare_data/a384bf5d-db92-421e-926d-bfb99a6b4b28/images/b7d6cc07-d1f1-44b3-b3c0-7067ec7056a3"}]
"""


def test_rhev_data_center():
    files = ls_dc.RhevDataCenter(context_wrap(RHEV_DATA_CENTER_INSIGHTS))
    assert len(files.data) == 4
    assert len(files.incorrect_volume_ownership) == 1
    assert files.incorrect_volume_ownership[0]['path'] == '/rhev/data-center/mnt/host1.example.com:_nfsshare_data/a384bf5d-db92-421e-926d-bfb99a6b4b28/images/b7d6cc07-d1f1-44b3-b3c0-7067ec7056a3/4d6e5dea-995f-4a4e-b487-0f70361f6137'
    assert files.incorrect_volume_ownership[0]['name'] == 'root'
    assert files.incorrect_volume_ownership[0]['group'] == 'root'


def test_no_data():
    with pytest.raises(SkipException) as ex:
        ls_dc.RhevDataCenter(context_wrap(''))
    assert 'No files found with incorrect ownership.' in str(ex)


def test_rhev_data_center_doc_examples():
    failed, total = doctest.testmod(
        ls_dc,
        globs={'rhev_dc': ls_dc.RhevDataCenter(context_wrap(RHEV_DATA_CENTER_INSIGHTS))}
    )
    assert failed == 0



from .. import parser, CommandParser
from . import parse_delimited_table
from insights.specs import Specs


@parser(Specs.route)
class Route(CommandParser):
    """Class to parse the ``route -n`` command

    Attributes:
        data (list): A list of dicts likes
            [
                {
                    "Use": "0",
                    "Iface": "eth0",
                    "Metric": "0",
                    "Destination": "10.66.208.0",
                    "Genmask": "255.255.255.0",
                    "Flags": "U",
                    "Ref": "0",
                    "Gateway": "0.0.0.0"
                },
                ...
            ]
    Usages:
        ru  = Route(...)
        if "10.66.20.0" in ru: # check if specified destination is in the route
            ...

    ---Sample---
    Kernel IP routing table
    Destination     Gateway         Genmask         Flags Metric Ref    Use Iface
    10.66.208.0     0.0.0.0         255.255.255.0   U     0      0        0 eth0
    169.254.0.0     0.0.0.0         255.255.0.0     U     1002   0        0 eth0
    0.0.0.0         10.66.208.254   0.0.0.0         UG    0      0        0 eth0
    """

    def parse_content(self, content):
        # heading_ignore is first line we _don't_ want to ignore...
        self.data = parse_delimited_table(content, heading_ignore=['Destination'])

    def __contains__(self, dest):
        return any(dest == line['Destination'] for line in self.data)

    def __iter__(self):
        for row in self.data:
            yield row



"""
Scheduler - file ``/sys/block/*/queue/scheduler``
=================================================

This parser parses the content from scheduler files. It stores available
values and also current selection for every device.

Sample content from schduler file:

    noop deadline [cfq]

Examples:
    >>> type(scheduler_obj)
    <class 'insights.parsers.scheduler.Scheduler'>
    >>> scheduler_obj.data
    {'sda': '[cfq]'}
    >>> scheduler_obj.device
    'sda'
    >>> scheduler_obj.schedulers
    ['noop', 'deadline', 'cfq']
    >>> scheduler_obj.active_scheduler
    'cfq'

"""

import re

from insights.specs import Specs
from .. import parser, get_active_lines, Parser


@parser(Specs.scheduler)
class Scheduler(Parser):
    """
    This class provides parsing for content of ``/sys/block/*/queue/scheduler``
    files.

    Attributes:
        device (str): Block device name
        schedulers (list): A list of available schedulers
        active_scheduler (str): An active scheduler
        data (dict): A dictionary with block device name as a key and an active scheduler as
            a value.
    """
    ACTIVE_SCHEDULER_PATTERN = re.compile(r'\[(.*)]')

    def parse_content(self, content):
        self.device = None
        self.schedulers = []
        self.active_scheduler = None
        self.data = {}  # Legacy value to keep backwards compatibility

        self.device = self.file_path.split('/')[3]
        for line in get_active_lines(content):
            r = self.ACTIVE_SCHEDULER_PATTERN.search(line)
            if r:
                self.active_scheduler = r.group(1)

            self.schedulers = line.replace('[', '').replace(']', '').split()

        # Set legacy values
        if self.active_scheduler:
            self.data = {self.device: '[' + self.active_scheduler + ']'}



"""
Cobbler modules configuration - file ``/etc/cobbler/modules.conf``
==================================================================
"""
from insights.core import IniConfigFile
from insights.core.plugins import parser
from insights.specs import Specs


@parser(Specs.cobbler_modules_conf)
class CobblerModulesConf(IniConfigFile):
    """
    The Cobbler modules configuration lists a set of services, and typically
    sets the module that provides that service.

    Sample input::

        [authentication]
        module = authn_spacewalk

        [authorization]
        module = authz_allowall

        [dns]
        module = manage_bind

        [dhcp]
        module = manage_isc

    Examples:

        >>> type(conf)
        <class 'insights.parsers.cobbler_modules_conf.CobblerModulesConf'>
        >>> conf.get('authentication', 'module')
        'authn_spacewalk'
        >>> conf.get('dhcp', 'module')
        'manage_isc'
    """
    pass



from insights.parsers.xinetd_conf import XinetdConf
from insights.tests import context_wrap


XINETD_CONF_0 = """
defaults
{
}

includedir /etc/xinetd.d
"""

XINETD_CONF_1 = """
defaults
{
# The next two items are intended to be a quick access place to
# temporarily enable or disable services.
#
#	enabled		=
#	disabled	=
        enabled         =

# Define general logging characteristics.
        log_type	= SYSLOG daemon info
        log_on_failure	= HOST
        log_on_success	= PID HOST DURATION EXIT

# Define access restriction defaults
#
#	no_access	=
#	only_from	=
#	max_load	= 0
        cps		= 50 10
        instances	= 50
        per_source	= 10

# Address and networking defaults
#
#	bind		=
#	mdns		= yes
        v6only		= no

# setup environmental attributes
#
#	passenv		=
        groups		= yes
        umask		= 002

# Generally, banners are not used. This sets up their global defaults
#
#	banner		=
#	banner_fail	=
#	banner_success	=
}

includedir /etc/xinetd.d
"""

XINETD_CONF_2_BAD = """
defaults {
        umask		= 002
}

includedir /etc/xinetd.d
"""

XINETD_CONF_3_BAD = """
defaults
{
        umask		= 002
}

balabala...

includedir /etc/xinetd.d
"""

XINETD_CONF_4_BAD = """
defaults {
        umask		002
}

includedir /etc/xinetd.d
"""

XINETD_CONF_5 = """
defaults
{
        umask		= 002
}

includedir /etc/xinetd.d/abc
"""
XINETD_CONF_6 = """
defaults
{
        umask		= 002
}
"""

XINETD_D_TFTP = """
service tftp
{
        socket_type             = dgram
        protocol                = udp
        wait                    = yes
        user                    = root
        server                  = /usr/sbin/in.tftpd
        server_args             = -s /var/lib/tftpboot
        disable                 = yes
        per_source              = 11
        cps                     = 100 2
        flags                   = IPv4
}
"""

CONF_PATH = '/etc/xinetd.conf'
D_TFTP_PATH = '/etc/xinetd.d/tftp'


def test_XinetdConf_0():
    xinetd_conf = XinetdConf(context_wrap(XINETD_CONF_0, path=CONF_PATH))
    data = xinetd_conf.data
    assert xinetd_conf.is_valid
    assert xinetd_conf.is_includedir
    assert data.get("includedir") == "/etc/xinetd.d"
    assert data.get("defaults") == {}


def test_XinetdConf_1():
    xinetd_conf = XinetdConf(context_wrap(XINETD_CONF_1, path=CONF_PATH))
    data = xinetd_conf.data
    assert xinetd_conf.is_valid
    assert xinetd_conf.is_includedir
    assert data.get("includedir") == "/etc/xinetd.d"
    assert data.get("defaults") == {
        'enabled': '',
        'v6only': 'no',
        'log_on_failure': 'HOST',
        'umask': '002',
        'log_on_success': 'PID HOST DURATION EXIT',
        'instances': '50',
        'per_source': '10',
        'groups': 'yes',
        'cps': '50 10',
        'log_type': 'SYSLOG daemon info'
    }
    assert xinetd_conf.file_name == 'xinetd.conf'
    assert xinetd_conf.file_path == CONF_PATH


def test_XinetdConf_tftp():
    d_tftp = XinetdConf(context_wrap(XINETD_D_TFTP, path=D_TFTP_PATH))
    data = d_tftp.data
    assert d_tftp.is_valid
    assert not d_tftp.is_includedir
    assert data.get("includedir") is None
    assert data.get("tftp") == {
        'protocol': 'udp',
        'socket_type': 'dgram',
        'server': '/usr/sbin/in.tftpd',
        'server_args': '-s /var/lib/tftpboot',
        'disable': 'yes',
        'flags': 'IPv4',
        'user': 'root',
        'per_source': '11',
        'cps': '100 2',
        'wait': 'yes'
    }
    assert d_tftp.file_name == 'tftp'
    assert d_tftp.file_path == D_TFTP_PATH


def test_XinetdConf_2():
    xinetd_conf = XinetdConf(context_wrap(XINETD_CONF_2_BAD, path=CONF_PATH))
    assert not xinetd_conf.is_valid


def test_XinetdConf_3():
    xinetd_conf = XinetdConf(context_wrap(XINETD_CONF_3_BAD, path=CONF_PATH))
    assert not xinetd_conf.is_valid


def test_XinetdConf_4():
    xinetd_conf = XinetdConf(context_wrap(XINETD_CONF_4_BAD, path=CONF_PATH))
    assert not xinetd_conf.is_valid


def test_XinetdConf_5():
    xinetd_conf = XinetdConf(context_wrap(XINETD_CONF_5, path=CONF_PATH))
    data = xinetd_conf.data
    assert xinetd_conf.is_valid
    assert not xinetd_conf.is_includedir
    assert data.get("includedir") == "/etc/xinetd.d/abc"
    assert data.get("defaults") == {'umask': '002'}



"""
HeatConf - file ``/etc/heat/heat.conf``
=======================================
"""
from insights.core import IniConfigFile
from insights.core.plugins import parser
from insights.specs import Specs


@parser(Specs.heat_conf)
class HeatConf(IniConfigFile):
    """
    This module provides plugins access to the heat.conf information.

    Typical content of the ``heat.conf`` is::

        [DEFAULT]
        heat_metadata_server_url = http://172.16.0.11:8000
        heat_waitcondition_server_url = http://172.16.0.11:8000/v1/waitcondition
        heat_watch_server_url =http://172.16.0.11:8003
        stack_user_domain_name = heat_stack
        stack_domain_admin = heat_stack_domain_admin
        stack_domain_admin_password = *********
        auth_encryption_key = V48p9fRZzWSRgjE96e2I1oGwn216xgqf
        log_dir = /var/log/heat
        notification_driver=messaging

        [clients_keystone]
        auth_uri =http://192.0.2.18:35357

    Usage of this parser is similar to others that use the ``IniConfigFile`` base
    class.

    Examples:
        >>> type(conf)
        <class 'insights.parsers.heat_conf.HeatConf'>
        >>> 'DEFAULT' in conf
        True
        >>> conf.get('clients_keystone', 'auth_uri')
        'http://192.0.2.18:35357'
    """
    pass



import doctest
import pytest
from insights.parsers import sys_bus, ParseException
from insights.parsers.sys_bus import CdcWDM
from insights.tests import context_wrap


SYS_DEVICE_USAGE = """
1
""".strip()

SYS_DEVICE_USAGE_EMPTY = """

""".strip()

SYS_DEVICE_USAGE_INVALID = """
not valid content
""".strip()


def test_netstat_doc_examples():
    env = {
        'device_usage': CdcWDM(context_wrap(SYS_DEVICE_USAGE)),
    }
    failed, total = doctest.testmod(sys_bus, globs=env)
    assert failed == 0


def test_bond_dynamic_lb_class():
    device_usage = CdcWDM(context_wrap(SYS_DEVICE_USAGE))
    assert device_usage.device_usage_cnt == 1
    assert device_usage.device_in_use is True


def test_class_exceptions():
    with pytest.raises(ParseException) as exc:
        device_usage = CdcWDM(context_wrap(SYS_DEVICE_USAGE_EMPTY))
        assert device_usage is None
    assert 'Invalid Content!' in str(exc)
    with pytest.raises(ParseException) as exc:
        device_usage = CdcWDM(context_wrap(SYS_DEVICE_USAGE_INVALID))
        assert device_usage is None
    assert 'Invalid Content!' in str(exc)



"""
Yum List Command
================

The parsers contains in this module are:

YumListInstalled - Command ``yum list installed``
-------------------------------------------------

YumListAvailable - Command ``yum list available``
-------------------------------------------------

"""
from collections import defaultdict

from insights import CommandParser, parser, SkipComponent
from insights.specs import Specs
from insights.parsers.installed_rpms import InstalledRpm, RpmList


class YumListRpm(InstalledRpm):
    """
    The same as :py:class:`insights.parsers.installed_rpms.InstalledRpm` but
    with an additional ``.repo`` attribute.
    """
    def __init__(self, data):
        self.repo = None
        """str: yum / dnf repository name, if available."""

        super(YumListRpm, self).__init__(data)


class YumListBase(CommandParser, RpmList):
    """
    Base class for the ``yum list [installed|available]`` commands.  Each line
    is parsed and stored in a ``YumListRpm`` object.

    .. note::

        ``YumListBase`` shares the :py:class:`insights.parsers.installed_rpms.RpmList`
        interface with :py:class:`insights.parsers.installed_rpms.InstalledRpms`.
        The only difference is ``YumListBase`` takes the output of ``yum list`` as
        its source data, and the :py:class:`YumListRpm` instances it produces
        contain a ``.repo`` attribute.
    """

    def __init__(self, context, package_status):
        self.expired_cache = False
        """bool: Indicates if the yum repo cache is expired."""

        self.package_status = package_status
        """str: Indicates if the list is of installed or available packages."""

        super(YumListBase, self).__init__(context)

    def _find_start(self, content):
        for i, c in enumerate(content):
            if 'Repodata is over 2 weeks old' in c:
                self.expired_cache = True
            elif c == self.package_status + " Packages":
                break
        return i + 1

    def _get_rows(self, content):
        """
        Yields:
            a list per row of the following form:

            .. code:: python

                [
                    <name.arch>,
                    <[epoch:]version-release>,
                    <repo or @installed-from-repo>
                ]
        """
        start = self._find_start(content)
        if start == len(content):
            raise SkipComponent()

        # join hanging wrapped lines together into a single line.
        # see https://bugzilla.redhat.com/show_bug.cgi?id=584525
        cur = []
        for line in content[start:]:
            if not cur:
                cur.append(line.strip())
            elif line.startswith(" "):
                cur.append(line.strip())
            else:
                yield " ".join(cur).split()
                cur = [line.strip()]

        if cur:
            yield " ".join(cur).split()

    def _make_record(self, package, ver_rel, repo):
        """
        Given the fields of a ``yum list`` row, return a dictionary
        of name, version, release, epoch, arch, and repo.
        """
        name, _, arch = package.rpartition(".")
        repo = repo.lstrip("@")

        # Kept as string in InstalledRpm. Duplicating here for consistency.
        epoch = "0"
        if ":" in ver_rel:
            epoch, ver_rel = ver_rel.split(":", 1)
        version, release = ver_rel.split("-")

        # This is special cased for InstalledRpm. Duplicating here for
        # consistency.
        if name.startswith('oracleasm') and name.endswith('.el5'):
            name, version2 = name.split('-', 1)
            version = version2 + '-' + version

        return {"name": name, "version": version, "release": release, "epoch":
                epoch, "arch": arch, "repo": repo}

    def _unknown_row(self, row):
        """
        Heuristic to tell us we've hit the bottom of the Installed
        Packages stanza.
        """
        return len(row) != 3 or row[:2] == ["Loaded", "plugins:"]

    def parse_content(self, content):
        """
        ``yum list`` output is basically tabular with an ignorable
        set of rows at the top and a line "Installed Packages" that designates
        the following rows as data. Each column has a maximum width, and if any
        column overflows, the following columns wrap to the next line and
        indent to their usual starting positions. It's also possible for the
        data rows to be followed by more lines that should be ignored. Since
        ``yum list`` is for human consumption, the footer lines can be
        syntactically ambiguous with data lines. We use heuristics to check for
        an invalid row to signal the end of data.
        """

        packages = defaultdict(list)
        for row in self._get_rows(content):
            if self._unknown_row(row):
                break
            rec = self._make_record(*row)
            packages[rec["name"]].append(YumListRpm(rec))
        self.packages = dict(packages)


@parser(Specs.yum_list_installed)
class YumListInstalled(YumListBase):
    """
    The ``YumListInstalled`` class parses the output of the ``yum list installed``
    command. Each line is parsed and stored in a ``YumListRpm`` object.

    Sample input data::

        Repodata is over 2 weeks old. Install yum-cron? Or run: yum makecache fast
        Loaded plugins: product-id, search-disabled-repos, subscription-manager
        Installed Packages
        GConf2.x86_64                    3.2.6-8.el7             @rhel-7-server-rpms
        GeoIP.x86_64                     1.5.0-11.el7            @anaconda/7.3
        ImageMagick.x86_64               6.7.8.9-15.el7_2        @rhel-7-server-rpms
        NetworkManager.x86_64            1:1.4.0-17.el7_3        installed
        NetworkManager.x86_64            1:1.8.0-9.el7           installed
        NetworkManager-config-server.noarch
                                         1:1.8.0-9.el7           installed
        Uploading Enabled Repositories Report
        Loaded plugins: priorities, product-id, rhnplugin, rhui-lb, subscription-
                      : manager, versionlock

    Examples:
        >>> type(installed_rpms)
        <class 'insights.parsers.yum_list.YumListInstalled'>
        >>> 'GeoIP' in installed_rpms
        True
        >>> installed_rpms.get_max('GeoIP')
        0:GeoIP-1.5.0-11.el7
        >>> installed_rpms.expired_cache
        True
        >>> type(installed_rpms.get_max('GeoIP'))
        <class 'insights.parsers.yum_list.YumListRpm'>
        >>> rpm1 = installed_rpms.get_max('GeoIP')
        >>> rpm1.package == 'GeoIP-1.5.0-11.el7'
        True
        >>> rpm1.nvr == 'GeoIP-1.5.0-11.el7'
        True
        >>> rpm1.source
        >>> rpm1.name
        'GeoIP'
        >>> rpm1.version
        '1.5.0'
        >>> rpm1.release
        '11.el7'
        >>> rpm1.arch
        'x86_64'
        >>> rpm1.epoch
        '0'
        >>> from insights.parsers.yum_list import YumListRpm
        >>> rpm2 = YumListRpm.from_package('GeoIP-1.6.0-11.el7.x86_64')
        >>> rpm1 == rpm2
        False
        >>> rpm1 > rpm2
        False
        >>> rpm1 < rpm2
        True
    """
    def __init__(self, context):
        super(YumListInstalled, self).__init__(context, "Installed")


@parser(Specs.yum_list_available)
class YumListAvailable(YumListBase):
    """
    The ``YumListAvailable`` class parses the output of the ``yum list available``
    command. Each line is parsed and stored in a ``YumListRpm`` object.

    Input and usage examples are identical to ``YumListInstalled`` but with
    "Installed" replaced with "Available" wherever applicable.
    """
    def __init__(self, context):
        super(YumListAvailable, self).__init__(context, "Available")



"""
Custom datasources to get the semid of all the inter-processes.
"""

from insights.core.context import HostContext
from insights.core.plugins import datasource
from insights.specs import Specs
from insights.core.dr import SkipComponent


@datasource(Specs.ipcs_s, HostContext)
def semid(broker):
    """
    This datasource provides a list of the semid of all the inter-processes.

    Note:
        This datasource may be executed using the following command:

        ``insights cat --no-header ipcs_s_i``

    Sample output::

        [
            '65570', '98353', '98354'
        ]

    Returns:
        list: A list of the semid of all the inter-processes.
    """
    content = broker[Specs.ipcs_s].content
    results = set()
    for s in content:
        s_splits = s.split()
        # key        semid      owner      perms      nsems
        # 0x00000000 65536      apache     600        1
        if len(s_splits) == 5 and s_splits[1].isdigit():
            results.add(s_splits[1])
    if results:
        return list(results)
    raise SkipComponent



import doctest

import insights.combiners.rhsm_release as rhsm_release_module
from insights.combiners.rhsm_release import RhsmRelease
from insights.parsers.rhsm_releasever import RhsmReleaseVer
from insights.parsers.subscription_manager_release import SubscriptionManagerReleaseShow
from insights.tests import context_wrap

RHSM_RELEASE = """
{"releaseVer": "7.6"}
""".strip()

SUBSCRIPTION_MANAGER_RELEASE = """
Release: 7.2
""".strip()

SM_PARSER = SubscriptionManagerReleaseShow(context_wrap(SUBSCRIPTION_MANAGER_RELEASE))
RHSM_PARSER = RhsmReleaseVer(context_wrap(RHSM_RELEASE))


def test_with_rhsm():
    rhsm_release = RhsmRelease(RHSM_PARSER, None)
    assert rhsm_release.set == '7.6'
    assert rhsm_release.major == 7
    assert rhsm_release.minor == 6


def test_with_sub_mgr():
    rhsm_release = RhsmRelease(None, SM_PARSER)
    assert rhsm_release.set == '7.2'
    assert rhsm_release.major == 7
    assert rhsm_release.minor == 2


def test_with_both():
    rhsm_release = RhsmRelease(RHSM_PARSER, SM_PARSER)
    assert rhsm_release.set == '7.6'
    assert rhsm_release.major == 7
    assert rhsm_release.minor == 6


def test_doc_examples():
    env = {
        'rhsm_release': RhsmRelease(RHSM_PARSER, None),
    }
    failed, total = doctest.testmod(rhsm_release_module, globs=env)
    assert failed == 0



from insights.core import CommandParser
from insights.tests import context_wrap
from insights.core.plugins import ContentException
import pytest

CMF = "blah: Command not found"
NO_FILES_FOUND = "No files found for docker.service"
NO_SUCH_FILE = "/usr/bin/blah: No such file or directory"
NOT_A_DIRECTORY = "/etc/mail/spamassassin/channel.d: Not a directory"
MULTI_LINE = """
blah: Command not found
/usr/bin/blah: No such file or directory
""".strip()
MULTI_LINE_BAD = """
Missing Dependencies:
    At Least One Of:
        insights.specs.default.DefaultSpecs.aws_instance_type
        insights.specs.insights_archive.InsightsArchiveSpecs.aws_instance_type
""".strip()


class MockParser(CommandParser):
    def parse_content(self, content):
        self.data = content


def test_command_not_found():
    with pytest.raises(ContentException) as e:
        MockParser(context_wrap(CMF))
    assert "Command not found" in str(e.value)


def test_no_files_found():
    with pytest.raises(ContentException) as e:
        MockParser(context_wrap(NO_FILES_FOUND))
    assert "No files found for" in str(e.value)


def test_no_such_file_or_directory():
    with pytest.raises(ContentException) as e:
        MockParser(context_wrap(NO_SUCH_FILE))
    assert "No such file or directory" in str(e.value)


def test_not_a_directory():
    with pytest.raises(ContentException) as e:
        MockParser(context_wrap(NOT_A_DIRECTORY))
    assert "Not a directory" in str(e.value)


def test_multi_line():
    assert MULTI_LINE.split('\n') == MockParser(context_wrap(MULTI_LINE)).data


def test_multi_line_ng():
    with pytest.raises(ContentException) as e:
        MockParser(context_wrap(MULTI_LINE_BAD))
    assert "Missing Dependencies:" in str(e.value)



from collections import defaultdict
from insights.core import filters

from insights.parsers.ps import PsAux, PsAuxcww
from insights.specs import Specs
from insights.specs.default import DefaultSpecs

import pytest
import sys


def setup_function(func):
    if func is test_get_filter:
        filters.add_filter(Specs.ps_aux, "COMMAND")

    if func is test_get_filter_registry_point:
        filters.add_filter(Specs.ps_aux, "COMMAND")
        filters.add_filter(DefaultSpecs.ps_aux, "MEM")

    if func is test_filter_dumps_loads:
        filters.add_filter(Specs.ps_aux, "COMMAND")


def teardown_function(func):
    if func is test_get_filter:
        del filters.FILTERS[Specs.ps_aux]

    if func is test_get_filter_registry_point:
        del filters.FILTERS[Specs.ps_aux]
        del filters.FILTERS[DefaultSpecs.ps_aux]

    if func is test_filter_dumps_loads:
        del filters.FILTERS[Specs.ps_aux]

    if func is test_add_filter_to_parser:
        del filters.FILTERS[Specs.ps_aux]

    if func is test_add_filter_to_parser_patterns_list:
        del filters.FILTERS[Specs.ps_aux]


@pytest.mark.skipif(sys.version_info < (2, 7), reason='Playbook verifier code uses oyaml library which is incompatable with this test')
def test_filter_dumps_loads():
    r = filters.dumps()
    assert r is not None

    filters.FILTERS = defaultdict(set)
    filters.loads(r)

    assert Specs.ps_aux in filters.FILTERS
    assert filters.FILTERS[Specs.ps_aux] == set(["COMMAND"])


def test_get_filter():
    f = filters.get_filters(Specs.ps_aux)
    assert "COMMAND" in f

    f = filters.get_filters(DefaultSpecs.ps_aux)
    assert "COMMAND" in f


def test_get_filter_registry_point():
    s = set(["COMMAND", "MEM"])
    f = filters.get_filters(DefaultSpecs.ps_aux)
    assert f & s == s

    f = filters.get_filters(Specs.ps_aux)
    assert "COMMAND" in f
    assert "MEM" not in f


def test_add_filter_to_parser():
    filter_string = "bash"
    filters.add_filter(PsAux, filter_string)

    spec_filters = filters.get_filters(Specs.ps_aux)
    assert filter_string in spec_filters

    parser_filters = filters.get_filters(PsAux)
    assert not parser_filters


def test_add_filter_to_parser_patterns_list():
    filters_list = ["bash", "systemd", "Network"]
    filters.add_filter(PsAux, filters_list)

    spec_filters = filters.get_filters(Specs.ps_aux)
    assert all(f in spec_filters for f in filters_list)

    parser_filters = filters.get_filters(PsAux)
    assert not parser_filters


def test_add_filter_to_parser_non_filterable():
    filter_string = "bash"
    filters.add_filter(PsAuxcww, filter_string)

    spec_filters = filters.get_filters(Specs.ps_auxcww)
    assert not spec_filters

    parser_filters = filters.get_filters(PsAuxcww)
    assert not parser_filters


def test_add_filter_exception_not_filterable():
    with pytest.raises(Exception):
        filters.add_filter(Specs.ps_auxcww, "bash")


def test_add_filter_exception_raw():
    with pytest.raises(Exception):
        filters.add_filter(Specs.metadata_json, "[]")


def test_add_filter_exception_empty():
    with pytest.raises(Exception):
        filters.add_filter(Specs.ps_aux, "")



import string

from insights.parsr import (EOF, HangingString, InSet, LeftBracket, Lift,
        LineEnd, Literal, Many, OneLineComment, Opt, PosMarker, RightBracket,
        skip_none, String, WithIndent, WS, WSChar)
from insights.parsr.query import Directive, Entry, eq, Section


class Error(Exception):
    """Base class for iniparser exceptions."""
    def __init__(self, msg=''):
        self.message = msg
        Exception.__init__(self, msg)

    def __repr__(self):
        return self.message

    __str__ = __repr__


class NoOptionError(Error):
    """A requested option was not found."""
    def __init__(self, section, option):
        Error.__init__(self, "No option {0!r} in section: {1!r}".format(option, section))
        self.option = option
        self.section = section
        self.args = (option, section)


class NoSectionError(Error):
    """Raised when no section matches a requested option."""
    def __init__(self, section):
        Error.__init__(self, 'No section: {0!r}'.format(section))
        self.section = section
        self.args = section


def parse_doc(content, ctx, return_defaults=False, return_booleans=True):
    def to_directive(x):
        name, rest = x
        rest = [rest] if rest is not None else []
        return Directive(name=name.value.strip(), attrs=rest, lineno=name.lineno, src=ctx)

    def to_section(name, rest):
        return Section(name=name.value.strip(), children=rest, lineno=name.lineno, src=ctx)

    def apply_defaults(cfg, include_defaults):
        if "DEFAULT" not in cfg:
            return cfg

        defaults = cfg["DEFAULT"]
        not_defaults = cfg[~eq("DEFAULT")]
        for c in not_defaults:
            for d in defaults.grandchildren:
                if d.name not in c:
                    c.children.append(d)

        if not include_defaults:
            cfg.children = list(not_defaults)
        return cfg

    header_chars = (set(string.printable) - set(string.whitespace) - set("[]")) | set(" ")
    sep_chars = set("=:")
    key_chars = header_chars - sep_chars
    value_chars = set(string.printable) - set("\n\r")

    Yes = Literal("yes", True, ignore_case=True)
    No = Literal("no", False, ignore_case=True)
    Tru = Literal("true", True, ignore_case=True)
    Fals = Literal("false", False, ignore_case=True)
    if return_booleans:
        Boolean = ((Yes | No | Tru | Fals) & (WSChar | LineEnd)) % "Boolean"

    LeftEnd = (WS + LeftBracket + WS)
    RightEnd = (WS + RightBracket + WS)
    Header = (LeftEnd >> PosMarker(String(header_chars)) << RightEnd) % "Header"
    Key = WS >> PosMarker(String(key_chars)) << WS
    Sep = InSet(sep_chars, "Sep")

    if return_booleans:
        Value = WS >> (Boolean | HangingString(value_chars))
    else:
        Value = WS >> (HangingString(value_chars))

    KVPair = WithIndent(Key + Opt(Sep >> Value)) % "KVPair"
    Comment = (WS >> (OneLineComment("#") | OneLineComment(";")).map(lambda x: None))

    Line = Comment | KVPair.map(to_directive)
    Sect = Lift(to_section) * Header * Many(Line).map(skip_none)
    Doc = Many(Comment | Sect).map(skip_none)
    Top = Doc << WS << EOF

    res = Entry(children=Top(content), src=ctx)
    return apply_defaults(res, return_defaults)



"""
SshDConfig - file ``/etc/ssh/sshd_config``
==========================================

The ``ssh`` module provides parsing for the ``sshd_config``
file.  The ``SshDConfig`` class implements the parsing and
provides a ``list`` of all configuration lines present in
the file.

Sample input is provided in the *Examples*.

Examples:
    >>> sshd_config_input = '''
    ... #	$OpenBSD: sshd_config,v 1.93 2014/01/10 05:59:19 djm Exp $
    ...
    ... Port 22
    ... #AddressFamily any
    ... ListenAddress 10.110.0.1
    ... Port 22
    ... ListenAddress 10.110.1.1
    ... #ListenAddress ::
    ...
    ... # The default requires explicit activation of protocol 1
    ... #Protocol 2
    ... Protocol 1
    ... '''.strip()
    >>> from insights.tests import context_wrap
    >>> shared = {SshDConfig: SshDConfig(context_wrap(sshd_config_input))}
    >>> sshd_config = shared[SshDConfig]
    >>> 'Port' in sshd_config
    True
    >>> 'PORT' in sshd_config
    True
    >>> 'AddressFamily' in sshd_config
    False
    >>> sshd_config['port']
    ['22', '22']
    >>> sshd_config['Protocol']
    ['1']
    >>> [line for line in sshd_config if line.keyword == 'Port']
    [KeyValue(keyword='Port', value='22', kw_lower='port'), KeyValue(keyword='Port', value='22', kw_lower='port')]
    >>> sshd_config.last('ListenAddress')
    '10.110.1.1'
    >>> sshd_config.get_line('ListenAddress')
    'ListenAddress 10.110.1.1'
    >>> sshd_config.get_values('ListenAddress')
    ['10.110.0.1', '10.110.1.1']
    >>> sshd_config.get_values('ListenAddress', default='0.0.0.0')
    ['10.110.0.1', '10.110.1.1']
    >>> sshd_config.get_values('ListenAddress', join_with=',')
    '10.110.0.1,10.110.1.1'
"""
from collections import namedtuple
from .. import Parser, parser, get_active_lines
from insights.specs import Specs
import re

# optional whitespace, at least one non-whitespace (the keyword), at least one whitespace (space), a plus literal, anything
PLUS_PATTERN = re.compile(r'^\s*\S+\s+\+.*$')


@parser(Specs.sshd_config)
class SshDConfig(Parser):
    """Parsing for ``/etc/ssh/sshd_config`` file.

    Properties:
        lines (list): List of `KeyValue` namedtupules for each line in
            the configuration file.
        keywords (set): Set of keywords present in the configuration
            file, each keyword has been converted to lowercase.
    """

    KeyValue = namedtuple('KeyValue', ['keyword', 'value', 'kw_lower', 'line'])
    """namedtuple: Represent name value pair as a namedtuple with case ."""

    # Note about "+" options. In some openssh versions and for some keywords,
    # a "+" at the beginning of the value list signifies that the values are
    # added to the default list of values.
    # IT IS NOT DETECTED AND PARSED HERE.
    # `parse_content()` doesn't split the individual values because some
    # keywords don't have a list of values (just one string).
    # See the docstring in `line_uses_plus()` for more details.
    # Re: BZ#1697477
    # Config lines may also be delimited by `=`, and values may be quoted
    # with `"`. Here it is assumed that config lines are well-formed.
    def parse_content(self, content):
        self.lines = []
        for line in get_active_lines(content):
            line_splits = [s.strip() for s in re.split(r"[\s=]+", line, 1)]
            kw, val = line_splits[0], line_splits[1].strip('"') if \
                len(line_splits) == 2 else ''
            self.lines.append(self.KeyValue(
                kw, val, kw.lower(), line
            ))
        self.keywords = set([k.kw_lower for k in self.lines])

    def __contains__(self, keyword):
        """
        (bool) Is this keyword found in the configuration file?
        """
        return keyword.lower() in self.keywords

    def __iter__(self):
        """
        (iter) Yields a list of all the lines in the configuration file that
        actually contained a configuration option.
        """
        for line in self.lines:
            yield line

    def __getitem__(self, keyword):
        """
        (list) Get all values of this keyword.
        """
        kw = keyword.lower()
        if kw in self.keywords:
            return [kv.value for kv in self.lines if kv.kw_lower == kw]

    def get(self, keyword):
        """
        Get all declarations of this keyword in the configuration file.

        Returns:
            (list): a list of named tuples with the following properties:
                - ``keyword`` - the keyword as given on that line
                - ``value`` - the value of the keyword
                - ``kw_lower`` - the keyword converted to lower case
                - ``line`` - the complete line as found in the config file
        """
        kw = keyword.lower()
        return [kv for kv in self.lines if kv.kw_lower == kw]

    def get_values(self, keyword, default='', join_with=None, split_on=None):
        """
        Get all the values assigned to this keyword.

        Firstly, if the keyword is not found in the configuration file, the
        value of the ``default`` option is used (defaulting to ``''``).

        Then, if the ``join_with`` option is given, this string is used to
        join the values found on each separate definition line.  Otherwise,
        each separate definition line is returned as a string.

        Finally, if the ``split_on`` option is given, this string is used to
        split the combined string above into a list.  Otherwise, the combined
        string is returned as is.
        """
        kw = keyword.lower()
        if kw in self.keywords:
            vals = [kv.value for kv in self.lines if kv.kw_lower == kw]
        else:
            vals = [default]
        # Should we join them together?
        if join_with is None:
            return vals
        # OK, join them together.
        val_str = join_with.join(vals)
        # Should we now split them?
        if split_on is None:
            return val_str
        else:
            return val_str.split(split_on)

    def get_line(self, keyword, default=''):
        """
        (str): Get the line with the last declarations of this keyword in the
        configuration file, optionally pretending that we had a line with the
        default value and a comment informing the user that this was a
        created default line.

        This is a hack, but it's commonly used in the sshd configuration
        because of the many lines that are commonly omitted because they
        have their default value.

        Parameters:
            keyword(str): Keyword to find
            default: optional value to supply if not found
        """
        lines = self.get(keyword)
        if lines:
            return lines[-1].line
        else:
            return "{keyword} {value}  # Implicit default".format(
                keyword=keyword, value=default
            )

    def line_uses_plus(self, keyword):
        """
        (union[bool, None]): Get the line with the last declarations of this
        keyword in the configuration file and returns whether the "+" option
        syntax is used.

        A "+" before the list of values denotes that the values are appended to
        the openssh defaults for the particular keyword.

        Returns True if the "+" is used, False if a line with the keyword was
        found but it doesn't use the "+" or None if such a line doesn't exist.

        Reasoning for the implementation:

        * The "+" means "added to the defaults".
        * The defaults depend on the particular openssh-server version and
          the parser doesn't know the version.
        * Therefore, it is infeasible to add the evaluation logic for "+"
          into `get_values()`.
        * Adding the logic into a combiner would mean a requirement that
          the combiner has a complete database of all defaults in all
          openssh-server version - infeasible again.
        * Not every keyword allows the use of "+" - it wouldn't make sense
          to parse "+" into `KeyValue` as it would make meaningless parsing
          for some options and meaningful for others. Building a database
          which options in which openssh-server versions support it or not
          would be infeasible.
        * The way chosen as the most sensible is this - `line_uses_plus()`
          used selectively by a rule for those options that support it, and
          it is up to the developer of such a rule to check it for those
          options manually.

        Parameters:
            keyword(str): Keyword to find
        """
        lines = self.get(keyword)
        if lines:
            found_line = lines[-1].line
            match = PLUS_PATTERN.match(found_line)
            return bool(match)
        else:
            return None

    def last(self, keyword, default=None):
        """
        str: Returns the value of the last keyword found in config.

        Parameters:
            keyword(str): Keyword to find
            default: optional value to supply if not found
        """
        entries = self.__getitem__(keyword)
        if entries:
            return entries[-1]
        else:
            return default



"""
MulticastQuerier - command ``find /sys/devices/virtual/net/ -name multicast_querier -print -exec cat {} \;``
============================================================================================================

This module provides processing for the output of the
``find -name multicast_querier ...`` command.

Sample output of this command looks like::

    /sys/devices/virtual/net/br0/bridge/multicast_querier
    0
    /sys/devices/virtual/net/br1/bridge/multicast_querier
    1
    /sys/devices/virtual/net/br2/bridge/multicast_querier
    0

The ``bri_val`` method is to return a dictionary contains bridge interface and
its multicast_querier value as the parsing result::

{'br0': 0, 'br1': 1, 'br2': 0}

Examples:
    >>> multicast_querier_content = '''
    ... /sys/devices/virtual/net/br0/bridge/multicast_querier
    ... 0
    ... /sys/devices/virtual/net/br1/bridge/multicast_querier
    ... 1
    ... /sys/devices/virtual/net/br2/bridge/multicast_querier
    ... 0
    ... '''.strip()
    >>> from insights.tests import context_wrap
    >>> from insights.parsers.multicast_querier import MulticastQuerier
    >>> shared = {MulticastQuerier: MulticastQuerier(context_wrap(multicast_querier_content))}
    >>> mq_results = MulticastQuerier(context_wrap(multicast_querier_content))
    >>> mq_results.bri_val
    {'br0': 0, 'br1': 1, 'br2': 0}
"""

from .. import parser, CommandParser
from insights.specs import Specs


@parser(Specs.multicast_querier)
class MulticastQuerier(CommandParser):
    """
    Parse the output of the command:

    `find /sys/devices/virtual/net/ -name multicast_querier -print -exec cat {} \;`

    Get a dictionary of "bridge interface" and the value of the parameter "multicast_querier"
    """
    @property
    def bri_val(self):
        return self._mapping

    def parse_content(self, content):
        self._mapping = {}
        for line in content:
            mq_val = ''
            if line.startswith('/sys/'):
                bri_iface = line.split('/')[5]
            else:
                mq_val = int(line.strip())
            self._mapping[bri_iface] = mq_val
        return



"""
FirewallD commands
==================

This module contains the following parsers:

FirewallCmdListALLZones - command ``/usr/bin/firewall-cmd --list-all-zones``
----------------------------------------------------------------------------
"""

from insights import parser, CommandParser
from insights.parsers import ParseException
from insights.specs import Specs


@parser(Specs.firewall_cmd_list_all_zones)
class FirewallCmdListALLZones(CommandParser):
    """
    Class for parsing the `/usr/bin/firewall-cmd --list-all-zones` command.

    Typical content of the command is::

        public (active)
            target: default
            icmp-block-inversion: no
            interfaces: eno1
            sources:
            services: dhcpv6-client ssh
            ports:
            protocols:
            masquerade: no
            forward-ports: port=80:proto=tcp:toport=12345:toaddr=
                port=81:proto=tcp:toport=1234:toaddr=
                port=83:proto=tcp:toport=456:toaddr=10.72.47.45
            source-ports:
            icmp-blocks:
            rich rules:
                rule family="ipv4" source address="10.0.0.0/24" destination address="192.168.0.10/32" port port="8080-8090" protocol="tcp" accept
                rule family="ipv4" source address="10.0.0.0/24" destination address="192.168.0.10/32" port port="443" protocol="tcp" reject
                rule family="ipv4" source address="192.168.0.10/24" reject
                rule family="ipv6" source address="1:2:3:4:6::" forward-port port="4011" protocol="tcp" to-port="4012" to-addr="1::2:3:4:7"


        trusted
            target: ACCEPT
            icmp-block-inversion: no
            interfaces:
            sources:
            services:
            ports:
            protocols:
            masquerade: no
            forward-ports:
            source-ports:
            icmp-blocks:
            rich rules:

    Examples:
        >>> type(zones)
        <class 'insights.parsers.firewall_cmd.FirewallCmdListALLZones'>
        >>> 'public' in zones.active_zones
        True
        >>> 'ACCEPT' in zones.zones['trusted']['target']
        True
        >>> zones.zones['public']['services']
        ['dhcpv6-client ssh']
        >>> 'port=83:proto=tcp:toport=456:toaddr=10.72.47.45' in zones.zones['public']['forward-ports']
        True


    Attributes:
        zones (dict): A dict of zone info

    Raises:
        ParseException: Raised when the output is in invalid format
    """

    def __init__(self, context):
        super(FirewallCmdListALLZones, self).__init__(context, ["firewalld is not running"])

    @property
    def active_zones(self):
        """Return a list of active zone name"""
        return [zone for zone, d in self.zones.items() if 'active' in d.get('_attributes', [])]

    def parse_content(self, content):
        self.zones = dict()
        zone_line = True
        zone_name = ''
        zone_attr_index = -1
        zone_attr_name = ''
        for line in content:
            line_strip = line.strip()
            if not line_strip:
                zone_line = True
                continue
            if zone_line:
                name_info = line_strip.split(None, 1)
                zone_name = name_info[0]
                self.zones[zone_name] = {}
                if len(name_info) > 1:
                    self.zones[zone_name]['_attributes'] = [i.strip() for i in name_info[1].strip('()').split(',')]
                zone_line = False
                zone_attr_index = -1
            else:
                current_index = len(line.rstrip()) - len(line_strip)
                zone_attr_index = current_index if zone_attr_index == -1 else zone_attr_index
                if current_index == zone_attr_index:
                    attrs = [i.strip() for i in line.split(':', 1)]
                    if len(attrs) != 2:
                        raise ParseException('Invalid format')
                    zone_attr_name, attr_value = attrs
                    attr_value = [attr_value] if attr_value else []
                    self.zones[zone_name][zone_attr_name] = attr_value
                else:
                    self.zones[zone_name][zone_attr_name].append(line_strip)



from insights.parsers import yum_updateinfo
from insights.tests import context_wrap
import doctest

YUM_UPDATEINFO_INPUT = """
FEDORA-2020-777f43619c bugfix         firefox-83.0-13.fc32.x86_64
FEDORA-2020-786c7010d2 bugfix         flatpak-libs-1.8.3-1.fc32.x86_64
FEDORA-2020-786c7010d2 bugfix         flatpak-selinux-1.8.3-1.fc32.noarch
FEDORA-2020-786c7010d2 bugfix         flatpak-session-helper-1.8.3-1.fc32.x86_64
FEDORA-2020-3e2cd487ea bugfix         fuse-overlayfs-1.3.0-1.fc32.x86_64
FEDORA-2020-79e9f139fe bugfix         gnome-control-center-3.36.5-1.fc32.x86_64
FEDORA-2020-79e9f139fe bugfix         gnome-control-center-filesystem-3.36.5-1.fc32.noarch
FEDORA-2020-352b61ce72 bugfix         iwl100-firmware-39.31.5.1-115.fc32.noarch
FEDORA-2020-352b61ce72 bugfix         iwl1000-firmware-1:39.31.5.1-115.fc32.noarch
FEDORA-2020-352b61ce72 bugfix         iwl105-firmware-18.168.6.1-115.fc32.noarch
FEDORA-2020-352b61ce72 bugfix         iwl135-firmware-18.168.6.1-115.fc32.noarch
FEDORA-2020-352b61ce72 Moderate/Sec.  iwl2000-firmware-18.168.6.1-115.fc32.noarch
"""

TEST_DATA = """
RSHA-2020-0001 security       firefox-83.0-13.fc32.x86_64
RHBA-2020-0002 bugfix         flatpak-libs-1.8.3-1.fc32.x86_64
RHEA-2020-0003 enhancement    flatpak-selinux-1.8.3-1.fc32.noarch
"""


def test_yum_updateinfo():
    info = yum_updateinfo.YumUpdateinfo(context_wrap(YUM_UPDATEINFO_INPUT))
    assert info is not None
    assert info.items[0]['advisory'] == 'FEDORA-2020-777f43619c'
    assert info.items[0]['type'] == 'bugfix'
    assert info.items[0]['package'] == 'firefox-83.0-13.fc32.x86_64'
    assert len(info.items) == 12


def test_yum_updateinfo_docs():
    env = {
        'updateinfo': yum_updateinfo.YumUpdateinfo(context_wrap(TEST_DATA))
    }
    failed, total = doctest.testmod(yum_updateinfo, globs=env)
    assert failed == 0



from insights.parsers.neutron_ovs_agent_log import NeutronOVSAgentLog
from insights.tests import context_wrap

from datetime import datetime

LOG = """
2016-11-09 14:39:25.348 3153 WARNING oslo_config.cfg [-] Option "rabbit_password" from group "oslo_messaging_rabbit" is deprecated for removal.  Its value may be silently ignored in the future.
2016-11-09 14:39:25.348 3153 WARNING oslo_config.cfg [-] Option "rabbit_userid" from group "oslo_messaging_rabbit" is deprecated for removal.  Its value may be silently ignored in the future.
2016-11-09 14:39:25.352 3153 INFO ryu.base.app_manager [-] loading app neutron.plugins.ml2.drivers.openvswitch.agent.openflow.native.ovs_ryuapp
2016-11-09 14:39:27.171 3153 INFO ryu.base.app_manager [-] loading app ryu.app.ofctl.service
2016-11-09 14:39:27.190 3153 INFO ryu.base.app_manager [-] loading app ryu.controller.ofp_handler
"""


def test_neutron_ovs_agent_log():
    log = NeutronOVSAgentLog(context_wrap(LOG))
    assert len(log.get("WARNING")) == 2
    assert len(list(log.get_after(datetime(2016, 11, 9, 14, 39, 26)))) == 2



from insights.parsers import hosts, SkipException
from insights.parsers.hosts import Hosts
from insights.tests import context_wrap
import doctest
import pytest

HOSTS_EXAMPLE = """
127.0.0.1 localhost localhost.localdomain localhost4 localhost4.localdomain4
::1 localhost localhost.localdomain localhost6 localhost6.localdomain6

# Comment
127.0.0.1 fte.foo.redhat.com ci.foo.redhat.com qa.foo.redhat.com stage.foo.redhat.com prod.foo.redhat.com # Comment at end of line

10.0.0.1 nonlocal.foo.redhat.com nonlocal2.bar.redhat.com
2001:db8::36    ipv6.example.com

# Hosts with tabs
192.0.2.1	bar.example.com	bar
2001:db8::38	atlas	atlas.example.com
""".strip() + "                  "

HOSTS_DOC = """
127.0.0.1 localhost localhost.localdomain localhost4 localhost4.localdomain4
::1 localhost localhost.localdomain localhost6 localhost6.localdomain6
# The same IP address can appear more than once, with different names
127.0.0.1 fte.example.com

10.0.0.1 nonlocal.example.com nonlocal2.fte.example.com
10.0.0.2 other.host.example.com # Comments at end of line are ignored
""".strip()

EXPECTED = {
    "127.0.0.1": [
        "localhost",
        "localhost.localdomain",
        "localhost4",
        "localhost4.localdomain4",
        "fte.foo.redhat.com",
        "ci.foo.redhat.com",
        "qa.foo.redhat.com",
        "stage.foo.redhat.com",
        "prod.foo.redhat.com"
    ],
    "::1": [
        "localhost",
        "localhost.localdomain",
        "localhost6",
        "localhost6.localdomain6"
    ],
    "10.0.0.1": [
        "nonlocal.foo.redhat.com",
        "nonlocal2.bar.redhat.com"
    ],
    '2001:db8::36': ['ipv6.example.com'],
    '192.0.2.1': ['bar.example.com', 'bar'],
    '2001:db8::38': ['atlas', 'atlas.example.com'],
}


def test_hosts():
    d = Hosts(context_wrap(HOSTS_EXAMPLE)).data
    assert len(d) == 6
    for key in ["127.0.0.1", "::1"]:
        assert key in d
        assert d[key] == EXPECTED[key]


def test_lines():
    l = Hosts(context_wrap(HOSTS_EXAMPLE)).lines
    assert len(l) == 7
    assert l[2]['ip'] == l[0]['ip'] == "127.0.0.1"
    assert l[-1]['names'] == ['atlas', 'atlas.example.com']
    assert l[3]['raw_line'] == '10.0.0.1 nonlocal.foo.redhat.com nonlocal2.bar.redhat.com'


def test_all_hosts():
    h = Hosts(context_wrap(HOSTS_EXAMPLE))
    all_names = h.all_names
    expected = set(EXPECTED["127.0.0.1"]) | set(EXPECTED["::1"]) | \
        set(EXPECTED["10.0.0.1"]) | set(EXPECTED['2001:db8::36']) | \
        set(EXPECTED["192.0.2.1"]) | set(EXPECTED['2001:db8::38'])
    assert all_names == expected
    assert len(h.all_ips) == 6
    assert h.all_ips == set(EXPECTED.keys())


def test_nonlocal():
    ob = Hosts(context_wrap(HOSTS_EXAMPLE))
    expected = {
        "10.0.0.1": EXPECTED["10.0.0.1"],
        '2001:db8::36': EXPECTED['2001:db8::36'],
        '192.0.2.1': EXPECTED['192.0.2.1'],
        '2001:db8::38': EXPECTED['2001:db8::38'],
    }
    assert ob.get_nonlocal() == expected


def test_ip_of():
    ob = Hosts(context_wrap(HOSTS_EXAMPLE))
    # One IP with multiple names
    assert ob.ip_of('localhost4.localdomain4') == '127.0.0.1'
    # IPv6 address, with single name
    assert ob.ip_of('ipv6.example.com') == '2001:db8::36'
    # No host found = None
    assert ob.ip_of('foo.example.com') is None


def test_exception():
    with pytest.raises(SkipException):
        Hosts(context_wrap(""))


def test_doc():
    env = {
            'hosts': Hosts(context_wrap(HOSTS_DOC)),
          }
    failed, total = doctest.testmod(hosts, globs=env)
    assert failed == 0



"""
PciRportTargetDiskPath
======================

Module for parsing the output of command ``find /sys/devices/ -maxdepth 10 -mindepth 9 -name stat -type f``.
"""

from insights.parsers import ParseException, SkipException
from insights import parser, CommandParser
from insights.specs import Specs


@parser(Specs.pci_rport_target_disk_paths)
class PciRportTargetDiskPaths(CommandParser):
    """
    Class for parsing ``find /sys/devices/ -maxdepth 10 -mindepth 9 -name stat -type f`` command output.

    Typical output of command ``find /sys/devices/ -maxdepth 10 -mindepth 9 -name stat -type f`` with
    the filter of 'block' looks like::

        /sys/devices/pci0000:00/0000:00:01.0/0000:04:00.6/host1/rport-1:0-1/target1:0:0/1:0:0:0/block/sdb/stat
        /sys/devices/pci0000:00/0000:00:01.0/0000:04:00.7/host2/rport-2:0-2/target2:0:0/2:0:0:0/block/sdc/stat
        /sys/devices/pci0000:00/0000:00:02.2/0000:02:00.0/host0/target0:1:0/0:1:0:0/block/sda/stat

    The Original data parsed looks like::

        [
               {
                   'target': 'target1:0:0',
                   'devnode': 'sdb',
                   'host_channel_id_lun': '1:0:0:0',
                   'pci_id': '0000:04:00.6',
                   'host': 'host1',
                   'rport': 'rport-1:0-1'
               },
               {
                   'target': 'target2:0:0',
                   'devnode': 'sdc',
                   'host_channel_id_lun': '2:0:0:0',
                   'pci_id': '0000:04:00.7',
                   'host': 'host2',
                   'rport': 'rport-2:0-2'
               },
               {
                   'target': 'target0:1:0',
                   'devnode': 'sda',
                   'host_channel_id_lun': '0:1:0:0',
                   'pci_id': '0000:02:00.0',
                   'host': 'host0',
               }
        ]

    Examples:
        >>> type(pd)
        <class 'insights.parsers.pci_rport_target_disk_paths.PciRportTargetDiskPaths'>
        >>> pd.pci_id
        ['0000:02:00.0', '0000:04:00.6', '0000:04:00.7']
        >>> pd.host
        ['host0', 'host1', 'host2']
        >>> pd.target
        ['target0:1:0', 'target1:0:0', 'target2:0:0']
        >>> pd.host_channel_id_lun
        ['0:1:0:0', '1:0:0:0', '2:0:0:0']
        >>> pd.devnode
        ['sda', 'sdb', 'sdc']

    Raises:
        ParseException: Input content is not available to parse
        SkipException: Input content is empty

    Attributes:
        path_list (list): the result parsed

    """

    @property
    def pci_id(self):
        """
        The all pci_id(s) from parsed content.

        Returns:
            list: pci id
        """
        return sorted(self.__pci_id_attributes)

    @property
    def devnode(self):
        """
        The all devicenode(s) from parsed content.

        Returns:
            list: device nodes
        """
        return sorted(self.__devnode_attributes)

    @property
    def host(self):
        """
        The all host(s) from parsed content.

        Returns:
            list: hosts
        """
        return sorted(self.__host_attributes)

    @property
    def rport(self):
        """
        The all rport(s) from parsed content.

        Returns:
            list: rports
        """
        return sorted(self.__rport_attributes)

    @property
    def target(self):
        """
        The all target(s) from parsed content.

        Returns:
            list: targets
        """
        return sorted(self.__target_attributes)

    @property
    def host_channel_id_lun(self):
        """
        The all host_channel_id_lun(s) from parsed content

        Returns:
            list: host_channel_id_lun
        """
        return sorted(self.__host_channel_id_lun_attributes)

    def parse_content(self, content):
        EMPTY = "Input content is empty"
        BADWD = "No useful data parsed in line: '{0}'"
        KEY_MAP = [
                # key,          required chars,   relative index
                {'key': 'host', 'chars': 'host', 'idx': 0},
                {'key': 'pci_id', 'chars': 'pci', 'idx': 2},
                {'key': 'rport', 'chars': 'rport-', 'idx': 0},
                {'key': 'target', 'chars': 'target', 'idx': 0},
                {'key': 'devnode', 'chars': 'block', 'idx': 1},
                {'key': 'host_channel_id_lun', 'chars': 'block', 'idx': -1},
        ]

        if not content:
            raise SkipException(EMPTY)

        pci = []
        self.__host_attributes = set()
        self.__rport_attributes = set()
        self.__target_attributes = set()
        self.__pci_id_attributes = set()
        self.__devnode_attributes = set()
        self.__host_channel_id_lun_attributes = set()

        for line in content:
            line_sp = list(filter(None, line.strip().split('/')))
            temp_pci = {}
            for i, l in enumerate(line_sp):
                for km in KEY_MAP:
                    if l.startswith(km['chars']):
                        temp_pci[km['key']] = line_sp[i + km['idx']]

            len_of_tp = len(temp_pci)
            if len_of_tp == 6 or (len_of_tp == 5 and 'rport' not in temp_pci):
                pci.append(temp_pci)
                self.__host_attributes.add(temp_pci['host'])
                if temp_pci.get('rport'):
                    self.__rport_attributes.add(temp_pci['rport'])
                self.__target_attributes.add(temp_pci['target'])
                self.__pci_id_attributes.add(temp_pci['pci_id'])
                self.__devnode_attributes.add(temp_pci['devnode'])
                self.__host_channel_id_lun_attributes.add(temp_pci['host_channel_id_lun'])
            else:
                raise ParseException(BADWD.format(line))
        self.path_list = pci



from insights.parsers.dnsmasq_config import DnsmasqConf
from insights.combiners.dnsmasq_conf_all import DnsmasqConfTree
from insights.tests import context_wrap

DNSMASQ_CONF_MAIN = """
# Listen on this specific port instead of the standard DNS port
# (53). Setting this to zero completely disables DNS function,
# leaving only DHCP and/or TFTP.
port=5353

no-resolv
domain-needed
no-negcache
max-cache-ttl=1
enable-dbus
dns-forward-max=5000
cache-size=5000
bind-dynamic
except-interface=lo
server=/in-addr.arpa/127.0.0.1
server=/cluster.local/127.0.0.1
# End of config
""".strip()

DNSMASQ_CONF_MAIN_CONF_DIR = """
# Listen on this specific port instead of the standard DNS port
# (53). Setting this to zero completely disables DNS function,
# leaving only DHCP and/or TFTP.
port=5353

no-resolv
domain-needed
no-negcache
max-cache-ttl=1
enable-dbus
dns-forward-max=5000
cache-size=5000
bind-dynamic
except-interface=lo
server=/in-addr.arpa/127.0.0.1
server=/cluster.local/127.0.0.1
conf-dir=/etc/dnsmasq.d
# End of config
""".strip()

DNSMASQ_CONF_MAIN_EXCLUDE_CONF_DIR = """
# Listen on this specific port instead of the standard DNS port
# (53). Setting this to zero completely disables DNS function,
# leaving only DHCP and/or TFTP.
port=5353

no-resolv
domain-needed
no-negcache
max-cache-ttl=1
enable-dbus
dns-forward-max=5000
cache-size=5000
bind-dynamic
except-interface=lo
server=/in-addr.arpa/127.0.0.1
server=/cluster.local/127.0.0.1
conf-dir=/etc/dnsmasq.d,.conf
# End of config
""".strip()

DNSMASQ_CONF_MAIN_INCLUDE_CONF_DIR = """
enable-dbus
dns-forward-max=5000
cache-size=5000
bind-dynamic
except-interface=lo
server=/in-addr.arpa/127.0.0.1
server=/cluster.local/127.0.0.1
conf-dir=/etc/dnsmasq.d,*.conf
# End of config
""".strip()

DNSMASQ_CONF_FILE_1 = """
server=/in-addr.arpa/127.0.0.1
log-queries
txt-record=example.com,"v=spf1 a -all"
""".strip()

DNSMASQ_CONF_FILE_2 = """
dns-forward-max=10000
no-resolv
domain-needed
no-negcache
max-cache-ttl=1
enable-dbus
""".strip()


def test_no_conf_dir():
    # no conf-dir
    dnsmasq1 = DnsmasqConf(context_wrap(DNSMASQ_CONF_MAIN, path="/etc/dnsmasq.conf"))
    dnsmasq2 = DnsmasqConf(context_wrap(DNSMASQ_CONF_FILE_1, path="/etc/dnsmasq.d/origin-dns.conf"))
    result = DnsmasqConfTree([dnsmasq1, dnsmasq2])
    assert "domain-needed" in result
    assert "log-queries" not in result
    assert len(result["server"]) == 2


def test_conf_dir():
    dnsmasq1 = DnsmasqConf(context_wrap(DNSMASQ_CONF_MAIN_CONF_DIR, path="/etc/dnsmasq.conf"))
    dnsmasq2 = DnsmasqConf(context_wrap(DNSMASQ_CONF_FILE_1, path="/etc/dnsmasq.d/origin-dns.conf"))
    result = DnsmasqConfTree([dnsmasq1, dnsmasq2])
    assert "txt-record" in result
    assert len(result["server"]) == 3

    dnsmasq1 = DnsmasqConf(context_wrap(DNSMASQ_CONF_MAIN_CONF_DIR, path="/etc/dnsmasq.conf"))
    dnsmasq2 = DnsmasqConf(context_wrap(DNSMASQ_CONF_FILE_2, path="/etc/dnsmasq.d/dns-forward-max.conf"))
    result = DnsmasqConfTree([dnsmasq1, dnsmasq2])
    assert len(result["dns-forward-max"]) == 2
    assert result["dns-forward-max"][-1].value == 10000


def test_exclude_conf_dir():
    dnsmasq1 = DnsmasqConf(context_wrap(DNSMASQ_CONF_MAIN_EXCLUDE_CONF_DIR, path="/etc/dnsmasq.conf"))
    dnsmasq2 = DnsmasqConf(context_wrap(DNSMASQ_CONF_FILE_1, path="/etc/dnsmasq.d/origin-dns.conf"))
    result = DnsmasqConfTree([dnsmasq1, dnsmasq2])
    assert "txt-record" not in result
    assert len(result["server"]) == 2


def test_include_conf_dir():
    dnsmasq1 = DnsmasqConf(context_wrap(DNSMASQ_CONF_MAIN_INCLUDE_CONF_DIR, path="/etc/dnsmasq.conf"))
    dnsmasq2 = DnsmasqConf(context_wrap(DNSMASQ_CONF_FILE_2, path="/etc/dnsmasq.d/1id-dns.conf"))
    result = DnsmasqConfTree([dnsmasq1, dnsmasq2])
    assert len(result["dns-forward-max"]) == 2



"""
ProcEnviron - File ``/proc/<PID>/environ``
==========================================

Parser for parsing the ``environ`` file under ``/proc/<PID>``
directory.

"""

from insights import Parser, parser, LegacyItemAccess
from insights.parsers import SkipException, ParseException
from insights.specs import Specs


class ProcEnviron(Parser, LegacyItemAccess):
    """
    Base class for parsing the ``environ`` file under special ``/proc/<PID>``
    directory into a dictionaries with environment variable name as key and containing
    environment variable value.

    Typical content looks like::

        REGISTRIES=--add-registry registry.access.redhat.com \x00OPTIONS= --selinux-enabled       --signature-verification=False\x00DOCKER_HTTP_HOST_COMPAT=1\x00ADD_REGISTRY=--add-registry registry.access.redhat.com\x00PATH=/usr/libexec/docker:/usr/bin:/usr/sbin\x00PWD=/run/docker/libcontainerd/containerd/135240dbd15a834acb21d68867930917afcc84c5f006ba65004acd88dccab756/init\x00LANG=en_US.UTF-8\x00GOTRACEBACK=crash\x00DOCKER_NETWORK_OPTIONS= --mtu=1450\x00DOCKER_CERT_PATH=/etc/docker\x00SHLVL=0\x00DOCKER_STORAGE_OPTIONS=--storage-driver devicemapper --storage-opt dm.fs=xfs --storage-opt dm.thinpooldev=/dev/mapper/docker--vg-docker--pool --storage-opt dm.use_deferred_removal=true --storage-opt dm.use_deferred_deletion=true \x00

    Examples:
        >>> proc_environ['REGISTRIES']
        '--add-registry registry.access.redhat.com'
        >>> 'OPTIONS' in proc_environ
        True

    Raises:
        insights.parsers.SkipException: if the ``environ`` file is empty or doesn't exist.
        insights.parsers.ParseException: if the ``environ`` file content is incorrect.

    """

    def parse_content(self, content):
        if not content:
            raise SkipException("Empty output.")
        if len(content) != 1:
            raise ParseException("Incorrect content: '{0}'".format(content[-1]))

        self.data = {}
        for item in content[0].split('\x00'):
            if '=' in item:
                k, v = item.strip().split('=', 1)
                self.data[k] = v
            elif item:
                raise ParseException("Incorrect content: '{0}'".format(item))


@parser(Specs.openshift_fluentd_environ)
class OpenshiftFluentdEnviron(ProcEnviron):
    """
    Class for parsing the ``environ`` file of the ``fluentd`` process.
    """
    pass


@parser(Specs.openshift_router_environ)
class OpenshiftRouterEnviron(ProcEnviron):
    """
    Class for parsing the ``environ`` file of the ``openshift-route`` process.
    """
    pass
